from typing import Type, TypeVar, Optional, List, Literal
import swibots
from swibots.api.community.models import CommunityHeading


class HeadingMethods:

    async def get_headings(
        self: "swibots.ApiClient", community_id: str, additional: bool = False
    ) -> List[CommunityHeading]:
        """
        Get the headings of a community

        :param community_id: The ID of the community
        :param additional: Whether to get additional information about the headings
        """
        return await self.chat_service.headings.get_headings(community_id, additional)

    async def create_heading(
        self: "swibots.ApiClient",
        community_id: str,
        name: str,
        chat_id: str,
        heading_for: Literal["CHANNEL", "GROUP"] = "CHANNEL",
        heading_type: Literal["BLANK", "VALUE"] = "VALUE",
    ):
        """
        Create a heading in a community

        :param community_id: The ID of the community
        :param name: The name of the heading
        :param chat_id: The ID of the chat
        :param heading_for: The type of the heading
        :param heading_type: The type of the heading
        """
        return await self.chat_service.headings.create_heading(
            community_id, name, chat_id, heading_for, heading_type
        )

    async def delete_heading(self, community_id: str, heading_name: str):
        """
        Delete a heading from a community

        :param community_id: The ID of the community
        :param heading_name: The name of the heading to delete
        """
        return await self.chat_service.headings.delete_heading(
            community_id, heading_name
        )

    async def edit_heading(
        self,
        community_id: str,
        heading_name: str,
        new_heading_name: str,
        heading_type: Literal["BLANK", "VALUE"] = "VALUE",
    ):
        """
        Edit a heading in a community

        :param community_id: The ID of the community
        :param heading_name: The name of the heading to edit
        :param new_heading_name: The new name of the heading
        :param heading_type: The type of the heading
        """
        return await self.chat_service.headings.edit_heading(
            community_id, heading_name, new_heading_name, heading_type
        )

    async def rearrange_headings(
        self: "swibots.ApiClient",
        community_id: str,
        heading_names: List[str],
        subheading: str = ''
    ):
        """
        Rearrange headings in a community

        :param community_id: The ID of the community
        :param heading_names: List of heading names in the desired order
        :param subheading: Optional subheading name
        """
        return await self.chat_service.headings.rearrange_headings(
            community_id, heading_names, subheading
        )

    async def move_heading_content(
        self,
        community_id: str,
        heading_for: Literal["GROUP", "CHANNEL", "STORE", "WIDGET"],
        heading_type: Literal["BLANK", "VALUE"],
        type_id: str,
        updated_heading: str
    ):
        """
        Move content from one heading to another

        :param community_id: The ID of the community
        :param heading_for: The type of content being moved
        :param heading_type: The type of heading
        :param type_id: The ID of the content being moved
        :param updated_heading: The name of the heading to move content to
        """
        move_dto = {
            "communityId": community_id,
            "headingFor": heading_for,
            "headingType": heading_type,
            "typeId": type_id,
            "updatedHeading": updated_heading
        }
        request_dto = {"moveHeadingContentDto": [move_dto]}
        return await self.chat_service.headings.move_heading_content(request_dto)



from .delete_bot_info import DeleteBotInfo
from .get_bot_info import GetBotInfo
from .update_bot_info import UpdateBotInfo
from .game_methods import GameMethods
from .answer_callback_query import AnswerCallback


class BotMethods(DeleteBotInfo, GetBotInfo, UpdateBotInfo, GameMethods, AnswerCallback):
    pass



from typing import Type, TypeVar, Optional, List
import swibots
from swibots.api.chat.models import Message
from swibots.api.community.models import Group, Channel


class ForwardMessage:
    async def forward_message(
        self: "swibots.ApiClient",
        message_id: int | List[int],
        group_channel: Optional[Group | Channel | str] = None,
        user_id: Optional[int] = None,
    ) -> Message | List[Message]:
        """Forward a message

        Parameters:
            message (``~switch.api.chat.models.Message`` | ``int``): The message to forward
            group_channel (``~switch.api.community.models.Group`` | ``~switch.api.community.models.Channel`` | ``str``): The group/channel to forward the message to
            receiver_id (``str``): The receiver id to forward the message to

        Returns:
            ``~switch.api.chat.models.Message``: The forwarded message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be forwarded

        This functions does the same as :meth:`~switch.api.chat.controllers.MessageController.forward_message`.
        """
        return await self.chat_service.messages.forward_message(
            message_id, group_channel, user_id
        )



import logging
from typing import TYPE_CHECKING, Optional, List, Literal
from swibots.api.community.models import CommunityHeading
from swibots.api.common.models import User
from urllib.parse import urlencode

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)


class HeadingsController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def get_headings(
        self, community_id: str, additional: bool = False
    ) -> List[CommunityHeading]:
        """
        :param community_id: The ID of the community
        :return: The Heading object
        """
        query = urlencode(
            {
                "additional": additional,
                "communityId": community_id,
            }
        )
        response = await self.client.get(f"/headings?{query}")
        return self.client.build_list(CommunityHeading, response.data)

    async def create_heading(
        self,
        community_id: str,
        name: str,
        chat_id: str,
        heading_for: Literal["CHANNEL", "GROUP"] = "CHANNEL",
        heading_type: Literal["BLANK", "VALUE"] = "VALUE",
    ):
        query = urlencode(
            {
                "communityId": community_id,
                "heading": name,
                "id": chat_id,
                "headingFor": heading_for,
                "headingType": heading_type,
            }
        )
        response = await self.client.post(f"/headings?{query}")
        return response.data

    async def delete_heading(self, community_id: str, heading_name: str):
        query = urlencode(
            {
                "communityId": community_id,
                "headingName": heading_name,
            }
        )
        response = await self.client.post(f"/headings/delete-headings?{query}")
        return response.data


    async def edit_heading(
        self,
        community_id: str,
        heading_name: str,
        new_heading_name: str,
        heading_type: Literal["BLANK", "VALUE"] = "VALUE",
    ):
        query = urlencode(
            {
                "communityId": community_id,
                "headingName": heading_name,
                "newHeadingName": new_heading_name,
                "headingType": heading_type,
            }
        )
        response = await self.client.post(f"/headings/edit-headings?{query}")
        return response.data
    
    async def rearrange_headings(self,
                                 community_id: str,
                                 heading_names: List[str],
                                 subheading: str = ''
                                 ):
        body = {
            "communityId": community_id,
            "headingNames": heading_names,
            "subHeading": subheading
        }
        response = await self.client.post("/headings/rearrange-headings",
                                          data=body)
        return response.data

    async def move_heading_content(self, request_dto: dict):
        """
        Move content from one heading to another
        
        :param request_dto: Dictionary containing move heading content details
        :return: Response data from the API
        """
        response = await self.client.post("/headings/move-heading-content", 
                                        data=request_dto)
        return response.data


from typing import Type, TypeVar, List
import swibots
from swibots.api.chat.models import ADInfo


class AdMethods:
    async def get_all_ads(
        self: "swibots.ApiClient", limit: int = 100, page: int = 0
    ) -> List[ADInfo]:
        """Get all ads created by the user

        Parameters:
            page (``int``): The page number to offset.
            limit (``int``): The number of result to include.

        Returns:
            List[ADInfo]
        """
        return await self.chat_service.ads.get_all_ads(page=page, limit=limit)



from typing import List
import swibots
from swibots.api.chat.models import Message


class GetMessagesBetweenUsers:
    async def get_messages_between_users(
        self: "swibots.ApiClient",
        other_user_id: int,
        user_id: int = None,
        limit: int = 100,
        offset: int = 0,
    ) -> List[Message]:
        """Get messages between users

        Parameters:
            other_user_id (``int``): The other user id.
            user_id (``int``, *optional*): The user id. Defaults to the current user id.
            limit (``int``, *optional*): The maximum number of messages to retrieve. Defaults to 100.
            offset (``int``, *optional*): The offset. Defaults to 0.

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_messages_between_users`.
        """
        return await self.chat_service.messages.get_messages_between_users(
            other_user_id, user_id, limit, offset
        )



import swibots
from typing import List, Optional
from swibots.base import SwitchObject
from swibots.api.bot.models import BotCommand
from swibots.api.common.models import User
from swibots.utils.types import JSONDict


class Organization(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        community_ids: List[str] = None,
        thumbnail: str = None,
        created_by: int = None,
        description: str = None,
        email: str = None,
        follower: bool = None,
        followers: int = None,
        id: str = None,
        instagram: str = None,
        twitter: str = None,
        telegram: str = None,
        name: str = None,
        profile_pic: str = None,
        source_code_link: str = None,
        website: str = None,
        created_at: str = None,
        updated_at: str = None,
        **kwargs
    ):
        super().__init__(app)
        self.community_ids = community_ids
        self.profile_pic = profile_pic
        self.name = name
        self.telegram = telegram
        self.twitter = twitter
        self.id = id
        self.followers = followers
        self.thumbnail = thumbnail
        self.email = email
        self.description = description
        self.follower = follower
        self.created_by = created_by
        self.instagram = instagram
        self.source_code_link = source_code_link
        self.website = website
        self.updated_at = updated_at
        self.created_at = created_at

    def to_json(self) -> JSONDict:
        return {
            "communityIds": self.community_ids,
            "coverPic": self.thumbnail,
            "createdAt": self.created_at,
            "createdBy": self.created_by,
            "description": self.description,
            "email": self.email,
            "follower": self.follower,
            "followers": self.followers,
            "id": self.id,
            "instagram": self.instagram,
            "name": self.name,
            "profilePic": self.profile_pic,
            "sourceCodeLink": self.source_code_link,
            "telegram": self.source_code_link,
            "twitter": self.twitter,
            "updatedAt": self.updated_at,
            "websiteLink": self.website,
        }

    def from_json(self, data: JSONDict | None) -> "Organization":
        if data:
            self.id = data.get("id")
            self.twitter = data.get("twitter")
            self.telegram = data.get("telegram")
            self.thumbnail = data.get("coverPic")
            self.source_code_link = data.get("sourceCodeLink")
            self.community_ids = data.get("communityIds")
            self.profile_pic = data.get("profilePic")
            self.created_at = data.get("createdAt")
            self.updated_at = data.get("updatedAt")
            self.description = data.get("description")
            self.name = data.get("name")
            self.website = data.get("websiteLink")
            self.created_by = int(data.get("createdBy", 0))
        return self

    async def apps(self) -> List["OrgApp"]:
        return await self.app.get_organization_apps(self.id)

    async def followers(self) -> List[User]:
        return await self.app.get_organization_followers(self.id)


class OrgApp(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        commands: List[BotCommand] = None,
        created_at: str = None,
        org_id: str = None,
        name: str = None,
        role: str = None,
        id: str = None,
        status: str = None,
        **kwargs
    ):
        super().__init__(app, **kwargs)
        self.commands = commands
        self.created_at = created_at
        self.id = id
        self.name = name
        self.org_id = org_id
        self.role = role
        self.status = status

    def from_json(self, data: JSONDict | None) -> "OrgApp":
        if data:
            #            self.commands = BotCommand.build_from_json_list(data.get("commands"), self.app)
            self.org_id = data.get("orgId")
            self.name = data.get("name")
            self.status = data.get("status")
            self.role = data.get("role")
        return self

    def to_json(self) -> JSONDict:
        return {
            "status": self.status,
            "commands": self.commands,
            "name": self.name,
            "role": self.role,
            "orgId": self.org_id,
            "id": self.id,
        }



from typing import Optional
from swibots.api.common.models import User
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject
import swibots


class PrivateJoinResponse(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
    ):
        super().__init__(app)
        self.channel_id = None
        self.community_id = None
        self.group_id = None
        self.members = []

    def from_json(self, data: JSONDict = None) -> "PrivateJoinResponse":
        if data:
            self.channel_id = data.get("channelId")
            self.community_id = data.get("communityId")
            self.group_id = data.get("groupId")
            self.members = [
                User.build_from_json(member, self.app) for member in data.get("members")
            ]
        return self



from swibots.base import SwitchObject
from typing import List

from .message import Message
from swibots.api.common.models.user import User
import swibots


class GroupChatHistory(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        messages: List[Message] = None,
    ):
        super().__init__(app)
        self.messages = messages or []

    def from_json(self, data: dict) -> "GroupChatHistory":
        self.messages = Message.build_from_json_list(data, self.app)
        return self

    def to_json(self) -> dict:
        return {
            "message": [message.to_json() for message in self.messages],
        }



import re, httpx
import os, signal
import asyncio
import logging
import shutil
from contextlib import AbstractContextManager
from typing import List, Optional, Callable
import swibots
from contextlib import suppress
from pathlib import Path
from httpx import AsyncClient
from signal import signal as signal_fn, SIGINT, SIGTERM, SIGABRT
from io import BytesIO
from swibots.bots import Bot
from importlib import import_module
from swibots.errors import SwitchError, CancelError
from swibots.api.community.events import CommunityEvent
from swibots.api.chat.events import ChatEvent
from swibots.bots import BotContext, Decorators, BaseHandler
from swibots.api.bot.models import BotInfo, BotCommand
from swibots.api.common.models import User
from swibots.api.callback import AppBar, AppPage
from swibots.api.common.events import Event
from swibots.rate_limiter import AsyncRateLimiter
from swibots.utils import (
    DownloadProgress,
    IOClient,
    RestClient,
    DownloadProgressCallback,
)
from swibots.errors import TokenInvalidError
from swibots.api import ApiClient

log = logging.getLogger(__name__)
LoaderLog = logging.getLogger("loader")
# Signal number to name
signals = {
    k: v
    for v, k in signal.__dict__.items()
    if v.startswith("SIG") and not v.startswith("SIG_")
}


class Client(Decorators, AbstractContextManager, ApiClient):
    """Bot client

    This is the main class for interacting with the Switch BOT API.

    """

    def __init__(
        self,
        token: Optional[str] = None,
        email: Optional[str] = None,
        password: Optional[str] = None,
        bot_description: Optional[str] = None,
        plugins: dict = None,
        auto_update_bot: Optional[bool] = True,
        loop: asyncio.AbstractEventLoop = None,
        receive_updates: Optional[bool] = True,
        app_bar: Optional[AppBar] = None,
        is_app: Optional[bool] = False,
        home_callback: Optional[str] = None,
        preview: Optional[AppPage] = None,
        session_name: Optional[str] = None,
        **kwargs,
    ):
        """
        Initialize the client

        Args:
            token (:obj:`str`): The bot token (required for bot login).
            email (:obj:`str`): User email (only for user login).
            password (:obj:`str`): Password (only for user login).
            bot_description(:obj:`str`): The bot description.
            auto_update_bot(:obj:`bool`): Whether to automatically update the bot description and the registered commands.
            plugins(:obj:`dict`): plugin path to load, use as: dict(root="plugins")
            loop (:obj:`asyncio.AbstractEventLoop`): The asyncio loop to use (default: asyncio.get_event_loop()).

        """
        super().__init__()

        if email and password:
            auth_result = self.auth_service.login(email, password)
            token = auth_result.access_token

        if not token:
            raise TokenInvalidError(f"'token' for the bot can't be '{token}'")

        self.token = token
        self._user_type = Bot
        self._bot_info: BotInfo | None = None
        self.on_app_start = None
        self.on_app_stop = None
        self.on_chat_service_start = self._on_chat_service_start
        self.on_community_service_start = self._on_community_service_start
        self._handlers: List[BaseHandler] = []
        self._register_commands: List[BotCommand] = []
        self._bot_description = bot_description
        self.auto_update_bot = auto_update_bot
        self._loop = loop or asyncio.get_event_loop()
        self._upload_mode = kwargs.get("upload_method")

        self.user = self.auth_service.get_me_sync(user_type=self._user_type)
        self.name = self.user.name

        self.rate_limit = AsyncRateLimiter(
            storage_file=f"{session_name or self.user.id}.pkl"
        )

        # Add rate limit functions
        self.update_bot_commands = self.rate_limit.limit(
            "update_bot_commands", 10, 15 * 60
        )(self.update_bot_commands)

        if app_bar is None:
            app_bar = AppBar(self.name, left_icon=self.user.imageurl)
        self.app_bar = app_bar
        self._bot_id = None
        self._running = False
        self._user_type = Bot
        self.rest_client = RestClient()
        self.receive_updates = receive_updates
        self.plugins = plugins or dict()
        self.app_bar = app_bar
        self._is_app = is_app
        self._home_callback = home_callback
        self._app_preview = preview

    @property
    def bot(self) -> "swibots.bots.Bot":
        """
        The bot user.

            Returns:
                :obj:`swibots.bots.Bot`: The bot user.
        """
        return self.user

    @property
    def handlers(self) -> List[BaseHandler]:
        """
        Get the list of handlers.

        Returns:
            :obj:`List[BaseHandler]`: The list of handlers.
        """
        return self._handlers

    def load_path(self, path):
        base_name = os.path.basename(path)
        if base_name.startswith("__") or not base_name.endswith(".py"):
            return
        try:
            module_path = path[:-3].replace("\\", ".").replace("/", ".")

            return import_module(module_path)
        except Exception as er:
            LoaderLog.exception(er)

    def load_plugins(self):
        if not self.plugins:
            return
        plugins = self.plugins.copy()

        for option in ["include", "exclude"]:
            if plugins.get(option, []):
                plugins[option] = [
                    (i.split()[0], i.split()[1:] or None) for i in self.plugins[option]
                ]

        root = plugins["root"]
        include = plugins.get("include", [])
        exclude = plugins.get("exclude", [])

        count = 0

        if not include:
            for path in sorted(Path(root.replace(".", "/")).rglob("*.py")):
                module_path = ".".join(path.parent.parts + (path.stem,))
                try:
                    module = import_module(module_path)
                    count += 1
                except Exception as er:
                    log.exception(er)
        else:
            for path in include:
                module_path = root + "." + path

                try:
                    module = import_module(module_path)
                    count += 1
                except ImportError:
                    continue

                if "__path__" in dir(module):
                    continue

        if exclude:
            for path in exclude:
                module_path = root + "." + path

                try:
                    module = import_module(module_path)
                except ImportError:
                    continue

                if "__path__" in dir(module):
                    continue

        if count > 0:
            log.info(
                '[{}] Successfully loaded {} modules{} from "{}"'.format(
                    self.name, count, "s" if count > 1 else "", root
                )
            )
            return
        log.warning('[%s] No modules loaded from "%s"', self.name, root)

    def set_bot_commands(self, command: BotCommand | List[BotCommand]) -> "BotApp":
        """Add commands to the bot.
        (Commands will be added to the bot when the app starts, .start() method is called)

        Args:
            command (BotCommand | List[BotCommand]): List of commands to add
        """
        if isinstance(command, list):
            self._register_commands.extend(command)
        else:
            self._register_commands.append(command)
        return self

    def delete_bot_commands(self, command: BotCommand | List[BotCommand]) -> "Client":
        if isinstance(command, list):
            for cmd in command:
                self._register_commands.remove(cmd)
        else:
            self._register_commands.remove(command)
        return self

    def add_handler(self, handler: BaseHandler | List[BaseHandler]) -> "BotApp":
        if isinstance(handler, list):
            self.handlers.extend(handler)
        else:
            self.handlers.append(handler)
        return self

    def remove_handler(self, handler: BaseHandler | List[BaseHandler]) -> "BotApp":
        if not isinstance(handler, list):
            handler = [handler]
        for h in handler:
            self.handlers.remove(h)
        return self

    async def update_bot_commands(self):
        # get all app commands
        commands = self._register_commands or []
        description = self._bot_description or ""
        # register the commands
        self._bot_info = BotInfo(
            description=description,
            id=self._bot_id,
            commands=commands,
            preview=self._app_preview,
        )
        log.info("Updating bot commands...")
        self._bot_info = await self.update_bot_info(self._bot_info)
        log.info("Bot commands updated")

    async def _on_chat_service_start(self, _):
        await self.chat_service.subscribe_to_notifications(callback=self.on_chat_event)

    async def _on_community_service_start(self, _):
        await self.community_service.subscribe_to_notifications(
            callback=self.on_community_event
        )

    def _build_context(self, event: Event) -> BotContext:
        return BotContext(app=self, event=event)

    async def process_event(self, ctx: BotContext):
        for handler in self.handlers:
            try:
                await handler.handle(ctx)
            except Exception as e:
                log.exception(f"Error while processing event: {e}")
                raise e

    async def on_community_event(self, evt: CommunityEvent):
        if evt is not None and isinstance(evt, Event):
            await self.process_event(self._build_context(evt))

    async def on_chat_event(self, evt: ChatEvent):
        if evt is not None:
            await self.process_event(self._build_context(evt))

    async def handle_download(
        self,
        url: str,
        file_name: str,
        directory="downloads/",
        in_memory: bool = False,
        progress: DownloadProgressCallback = None,
        progress_args: tuple = (),
    ):
        if directory is None or directory == "":
            directory = "downloads/"
        if not in_memory:
            os.makedirs(directory, exist_ok=True)
        file_path = os.path.abspath(
            re.sub("\\\\", "/", os.path.join(directory, file_name))
        )
        file = BytesIO() if in_memory else open(file_path, "wb")

        d_progress = DownloadProgress(
            total=0,
            downloaded=0,
            file_name=file_name,
            client=IOClient(),
            url=url,
        )

        try:
            async with AsyncClient().stream("GET", url) as response:
                d_progress.total = int(response.headers["Content-Length"])
                d_progress.downloaded = response.num_bytes_downloaded
                d_progress.client = response
                d_progress.started = True
                async for chunk in response.aiter_bytes():
                    file.write(chunk)
                    d_progress.downloaded += len(chunk)
                    if progress:
                        await progress(d_progress, *progress_args)

        except BaseException as e:
            if not in_memory:
                file.close()
                os.remove(file_path)
            if isinstance(e, CancelError):
                return None
            if isinstance(e, asyncio.CancelledError):
                raise e
            raise e
        if in_memory:
            file.name = file_name
            return file
        else:
            file.close()
            return file_path

    async def _on_app_stop(self):
        await self.chat_service.stop()
        await self.community_service.stop()
        if self.on_app_stop is not None:
            await self.on_app_stop(self)
        for task in asyncio.all_tasks():
            task.cancel()

    async def _on_app_start(self):
        if self.on_app_start is not None:
            await self.on_app_start(self)

    async def start(self):
        """Starts the app"""
        try:
            if self._running:
                return
            self._running = True
            log.info("🚀 Starting app...")
            if self.user.id:
                log.info(
                    "Logged in as [%a][%s][%d]",
                    self.user.name,
                    self.user.user_name,
                    self.user.id,
                )
                self.name = self.user.user_name
                self._bot_id = self.user.id
            else:
                log.error("ERROR: Can't identify logged user.")

            if self._is_app or self._home_callback:
                await self.update_user_info(
                    user_info=User(
                        id=self.user.id,
                        is_app=self._is_app,
                        app_callback=self._home_callback,
                    )
                )

            await self.update_bot_commands()
            self.load_plugins()

            if self.receive_updates:
                try:
                    await self.chat_service.start()
                    if self.on_chat_service_start is not None:
                        await self.on_chat_service_start(self)
                except Exception as e:
                    log.exception(e)

                try:
                    await self.community_service.start()
                    if self.on_community_service_start is not None:
                        await self.on_community_service_start(self)
                except Exception as e:
                    log.exception(e)

            await self._on_app_start()

            log.info("🚀 App started!")

        except asyncio.CancelledError:
            self._running = False

    async def stop(self):
        if not self._running:
            return
        log.info("🛑 Stopping app...")
        await self._on_app_stop()
        self._running = False

    def __enter__(self):
        return self.start()

    def __exit__(self, *args):
        with suppress(ConnectionError):
            self.stop()

    async def __aenter__(self):
        return await self.start()

    async def __aexit__(self, *args):
        with suppress(ConnectionError):
            await self.stop()

    async def idle(self):
        task = None

        def signal_handler(signum, __):
            logging.info(f"Stop signal received ({signals[signum]}). Exiting...")
            task.cancel()

        for s in (SIGINT, SIGTERM, SIGABRT):
            signal_fn(s, signal_handler)

        while True:
            task = asyncio.create_task(asyncio.sleep(600))

            try:
                await task
            except asyncio.CancelledError:
                break

    def run(self, task: Callable = None):
        loop = asyncio.get_event_loop()
        run = loop.run_until_complete
        if task is not None:
            return run(task)
        else:
            try:
                run(self.start())
                run(self.idle())
                run(self.stop())
            except KeyboardInterrupt:
                run(self.stop())
            except asyncio.exceptions.CancelledError as er:
                log.debug(er)
            except Exception as e:
                log.exception(e)
                run(self.stop())

    def limit(self, key: str, times: int, seconds: int):
        pass


BotApp = Client



from enum import Enum
from swibots.base import SwitchObject
from swibots.utils.types import JSONDict
from typing import Dict, Any, List


class BottomBarTile(SwitchObject):
    def __init__(
        self,
        name: str,
        icon: str = "",
        selection_icon: str = "",
        dark_icon: str = "",
        dark_selection_icon: str = "",
        callback_data: str = "",
        selected: bool = False,
    ):
        self.name = name
        self.icon = icon
        self.selected_icon = selection_icon
        self.dark_icon = dark_icon or icon
        self.dark_selection_icon = dark_selection_icon or selection_icon
        self.selected = selected
        self.callback_data = callback_data

    def to_json(self):
        return {
            "name": self.name,
            "icon": self.icon,
            "selectionIcon": self.selected_icon,
            "selected": self.selected,
            "callbackData": self.callback_data,
            "darkIcon": self.dark_icon,
            "darkSelectedIcon": self.dark_selection_icon,
        }


class BottomBarType(Enum):
    DEFAULT = "default"
    TOPLINE = "topline"
    BOTTOMLINE = "bottomline"
    TOP_NOTCH = "top_notch"
    BOTTOM_NOTCH = "bottom_notch"


class BottomBar(SwitchObject):
    def __init__(
        self,
        options: List[BottomBarTile] = None,
        type: BottomBarType = BottomBarType.DEFAULT,
        theme_color: str = "",
    ):
        self.options = options
        self.type = type
        self.theme_color = theme_color

    def from_json(self, data) -> Any:
        if "BottomBar" in data:
            self.type = BottomBarType(data.get("bottomBarStyle"))
            self.options = [
                BottomBarTile().from_json(option) for option in data.get("bottomBar")
            ]
            self.theme_color = data.get("bottomBarColour")

        return self

    def to_json(self):
        data = {
            "bottomBar": [option.to_json() for option in self.options or []],
            "bottomBarStyle": self.type.value,
        }
        if self.theme_color:
            data["bottomBarColour"] = self.theme_color
        return data



import json
import logging
from typing import TYPE_CHECKING, List, Optional
from swibots.api.bot.models import BotInfo
from swibots.errors import SwitchError
from swibots.utils.types import JSONDict
from swibots.api.callback import AppPage

if TYPE_CHECKING:
    from swibots.api.bot import BotClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/bots"


class BotController:
    """Bot controller

    This controller is used to communicate with the bot endpoints.
    """

    def __init__(self, client: "BotClient"):
        self.client = client

    async def get_bot_info(self, bot_id: str) -> BotInfo:
        """Get bot info

        Parameters:
            bot_id (``str``): The bot id. Defaults to the current bot id.

        Returns:
            :obj:``~switch.api.bot.models.BotInfo``: The bot info
        """
        if bot_id is None:
            bot_id = self.client.user.id
        response = await self.client.get(BASE_PATH + "?botId=" + str(bot_id))
        return BotInfo.build_from_json(response.data)

    async def update_bot_info(self, bot_info: BotInfo) -> BotInfo:
        """Update bot info

        Parameters:
            bot_info (``~switch.api.bot.models.BotInfo``): The bot info to update

        Returns:
            :obj:``~switch.api.bot.models.BotInfo``: The bot info
        """
        data = bot_info.to_json_request()
        if "preview" in data:
            data["preview"] = json.dumps(data["preview"])
        response = await self.client.put(f"{BASE_PATH}/update", data=data)
        return BotInfo.build_from_json(response.data)

    async def delete_bot_info(self, bot_id: str) -> bool:
        """Delete bot info

        Parameters:
            bot_id (``str``): The bot id. Defaults to the current bot id.

        Returns:
            ``bool``: True if the bot was deleted
        """
        response = await self.client.delete(f"{BASE_PATH}/{bot_id}")
        return True

    async def answer_callback_query(
        self,
        callback_id: str,
        text: str,
        url: Optional[str] = None,
        message_id: Optional[int] = 0,
        show_alert: Optional[bool] = False,
        cache_time: Optional[int] = None,
        app_session_id: Optional[str] = None,
    ) -> bool:
        response = await self.client.post(
            f"{BASE_PATH}/callback/answer",
            data={
                "type": "callback",
                "callbackQueryId": callback_id,
                "messageId": str(message_id),
                "text": text,
                "url": url,
                "showAlerts": show_alert,
                "cacheTime": cache_time,
                "appSessionId": app_session_id,
            },
        )
        return response.data

    async def answer_ui_query(
        self, callback_id: str, message_id: int, callback: AppPage, app_session_id: str
    ) -> bool:
        default_bar = self.client.app.app_bar
        if not callback.app_bar and default_bar:
            callback.app_bar = default_bar
        data = callback.to_json_request()
        data.update(
            {
                "appSessionId": app_session_id,
                "callbackQueryId": callback_id,
                "messageId": message_id,
            }
        )
        response = await self.client.post(f"{BASE_PATH}/callback/answer", data=data)
        return response.data

    async def set_welcome(
        self,
        text: str = None,
        thumb: str = None,
        button: str = None,
        command: str = None,
    ):
        response = await self.client.post(
            f"{BASE_PATH}/set-intro-message",
            data={
                "botId": str(self.client.user.id),
                "welcomeImage": thumb,
                "welcomeText": text,
                "buttonName": button,
                "buttonCommand": command,
            },
        )
        return self.client.build_object(BotInfo, response.data)



import logging
from typing import TYPE_CHECKING, Optional, List
from swibots.api.common.models import User
from swibots.api.community.models import Quest, QuestCategory
from swibots.responses import CommunityQuestResponse
from urllib.parse import urlencode

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/quest"


class QuestsController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def create_quest(self, quest: Quest):
        response = await self.client.post(BASE_PATH, data=quest.to_json())
        return self.client.build_object(Quest, response.data)

    async def update_quest(self, quest: Quest):
        response = await self.client.put(BASE_PATH, data=quest.to_json())
        return self.client.build_object(Quest, response.data)

    async def get_quest(self, quest_id: int):
        return self.client.build_object(
            Quest, (await self.client.get(f"{BASE_PATH}/{quest_id}")).data
        )

    async def submit_quest(self, quest_id: int, community_id: str, description: str):
        response = await self.client.post(
            f"{BASE_PATH}/submitQuest",
            data={
                "questId": quest_id,
                "communityId": community_id,
                "description": description,
            },
        )
        return response.data

    async def approve_quest(
        self, community_id: str, quest_id: int, winner_id: int
    ) -> Quest:
        response = await self.client.post(
            f"{BASE_PATH}/approveQuest",
            data={
                "communityId": community_id,
                "questId": quest_id,
                "winnerMemberId": winner_id,
            },
        )
        return self.client.build_object(Quest, response.data)

    async def delete_quest(self, quest_id: str) -> bool:
        response = await self.client.delete(f"{BASE_PATH}/{quest_id}")
        return response.data

    async def get_quest_participants(self, quest_id: int):
        response = await self.client.get(
            f"{BASE_PATH}/getParticipants", data={"questId": quest_id}
        )
        return self.client.build_object(User, response.data)

    async def get_quests_by_community(self, community_id: str):
        response = await self.client.get(BASE_PATH, data={"communityId": community_id})
        return self.client.build_object(CommunityQuestResponse, response.data)

    # region

    async def get_quest_categories(self, community_id: str) -> List[QuestCategory]:
        return self.client.build_list(
            QuestCategory,
            (
                await self.client.get(
                    f"{BASE_PATH}/categories", data={"communityId": community_id}
                )
            ).data,
        )

    async def get_quest_category(self, category_id: str) -> QuestCategory:
        return self.client.build_object(
            QuestCategory,
            (
                await self.client.get(
                    f"{BASE_PATH}/category", data={"categoryId": category_id}
                )
            ).data,
        )

    async def create_quest_category(
        self, category_name: str, category_id: Optional[str] = None
    ) -> QuestCategory:
        response = await self.client.post(
            f"{BASE_PATH}/category",
            data={
                "categoryName": category_name,
                "categoryId": category_id or category_name,
            },
        )
        return self.client.build_object(QuestCategory, response.data)

    async def delete_quest_category(self, category_id: str) -> bool:
        response = await self.client.delete(
            f"{BASE_PATH}/category",
            data={
                "categoryId": category_id,
            },
        )
        return response.data

    # endregion



from typing import Optional
import swibots
from swibots.base.switch_object import SwitchObject

# from switch.api.community.models import Community, Group, Channel
# from switch.base.switch_object import SwitchObject
# from switch.types import EventType
# from switch.api.common.models.user import User
# from switch.utils.types import JSONDict


class Event(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        type: Optional["swibots.EventType"] = None,
        data: Optional[dict] = None,
        community_id: Optional[str] = None,
        community: Optional["swibots.Community"] = None,
        group_id: Optional[str] = None,
        group: Optional["swibots.Group"] = None,
        channel_id: Optional[str] = None,
        channel: Optional["swibots.Channel"] = None,
        action_by_id: Optional[int] = None,
        action_by: Optional["swibots.User"] = None,
    ):
        super().__init__(app)
        self.type = type
        self.data = data
        self.action_by_id = action_by_id
        self.action_by = action_by
        self.community_id = community_id
        self.community = community
        self.group_id = group_id
        self.group = group
        self.channel_id = channel_id
        self.channel = channel

    def from_json(self, data: swibots.JSONDict) -> "Event":
        if data is not None:
            details = data.get("details") or {}
            self.type = swibots.EventType(data.get("type", "").upper())
            self.action_by_id = int(data.get("actionById", 0))
            self.action_by = swibots.User.build_from_json(
                data.get("actionBy"), self.app
            )
            self.community_id = details.get("communityId")
            self.community = swibots.Community.build_from_json(
                details.get("community"), self.app
            )
            if not self.community_id and self.community:
                self.community_id = self.community.id
            self.group_id = details.get("groupId")
            self.group = swibots.Group.build_from_json(details.get("group"), self.app)
            self.channel_id = details.get("channelId")
            self.channel = swibots.Channel.build_from_json(
                details.get("channel"), self.app
            )
            self.data = details
        return self

    def to_json(self) -> swibots.JSONDict:
        return {
            "type": self.type,
            "details": self.data,
            "communityId": self.community_id,
            "community": self.community.to_json() if self.community else None,
            "groupId": self.group_id,
            "group": self.group.to_json() if self.group else None,
            "channelId": self.channel_id,
            "channel": self.channel.to_json() if self.channel else None,
            "actionById": self.action_by_id,
            "actionBy": self.action_by.to_json() if self.action_by else None,
        }



Byte = {"LF": "\x0A", "NULL": "\x00"}


class WsFrame:
    def __init__(self, command, headers, body):
        self.command = command
        self.headers = headers
        self.body = body

    def __str__(self):
        if self.command == Byte["LF"]:
            return self.command

        lines = [self.command]
        skip_content_length = "content-length" in self.headers
        if skip_content_length:
            del self.headers["content-length"]

        for name in self.headers:
            value = self.headers[name]
            lines.append("" + name + ":" + value)

        if self.body is not None and not skip_content_length:
            lines.append("content-length:" + str(len(self.body)))

        if self.body is not None:
            lines.append(Byte["LF"] + self.body)
        else:
            lines.append(Byte["LF"])

        return Byte["LF"].join(lines)

    @staticmethod
    def unmarshall_single(data):
        lines = data.split(Byte["LF"])
        command = lines[0].strip()
        if command == "":
            command = "PONG"
            return WsFrame(command, {}, None)
        headers = {}

        # get all headers
        i = 1
        while lines[i] != "":
            # get key, value from raw header
            (key, value) = lines[i].split(":")
            headers[key] = value
            i += 1

        # set body to None if body is none
        body = None if lines[i + 1] == Byte["NULL"] else lines[i + 1][:-1]

        return WsFrame(command, headers, body)

    @staticmethod
    def marshall(command, headers, body):
        return str(WsFrame(command, headers, body)) + Byte["NULL"]



from typing import Optional, List
from swibots.base.switch_object import SwitchObject
from swibots.utils.types import JSONDict


class AuthResult(SwitchObject):
    def __init__(
        self,
        access_token: Optional[str] = None,
        refresh_token: Optional[str] = None,
        user_id: Optional[int] = None,
        is_bot: Optional[bool] = False,
        roles: Optional[List[str]] = None,
        active: Optional[bool] = False,
    ):
        self.access_token = access_token
        self.refresh_token = refresh_token
        self.user_id = user_id
        self.is_bot = is_bot
        self.roles = roles
        self.active = active

    def from_json(self, data: Optional[JSONDict]) -> "AuthResult":
        super().from_json(data)
        if data is not None:
            self.access_token = data.get("access_token")
            self.refresh_token = data.get("refresh_token")
            self.user_id = data.get("user_id")
            self.is_bot = data.get("is_bot")
            self.roles = data.get("roles")
            self.active = data.get("active")
        return self



import logging
from typing import TYPE_CHECKING

from swibots.api.community.models import Role, RolePermission


if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/roles"


class RolesController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def get_roles(self, community_id: str):
        """Get all roles by community id"""
        response = await self.client.get(
            f"{BASE_PATH}/getAll?communityId={community_id}"
        )
        return self.client.build_list(Role, response.data.get("result"))

    async def add_role(
        self, community_id: str, role_colour: str, role_name: str
    ) -> Role:
        """Add role to the community"""
        role = Role(
            app=self.client.app,
            community_id=community_id,
            colour=role_colour,
            name=role_name,
        )
        await self.client.post(
            f"{BASE_PATH}/add?communityId={community_id}",
            data=role.to_json_request(),
        )
        return role

    async def update_role(
        self,
        community_id: str,
        role_id: int,
        role_colour: str = None,
        role_name: str = None,
    ) -> bool:
        """update role by community and role id"""
        await self.client.put(
            f"{BASE_PATH}/update",
            data=Role(
                app=self.client.app,
                colour=role_colour,
                community_id=community_id,
                name=role_name,
                id=role_id,
            ).to_json_request(),
        )
        return True

    async def delete_role(self, roleId: int):
        """delete role by id"""
        response = await self.client.delete(f"{BASE_PATH}/delete/{roleId}")
        return response.data.get("success")



import swibots

from typing import List
from swibots.api.community.models import Group


class GroupMethods:
    async def get_group(self: "swibots.ApiClient", id: str) -> Group:
        """Get a community by its ID

        Args:
            id (`str`): The ID of the group

        Returns:
            :obj:`swibots.api.community.models.Group`: The group object.

        Raises:
            :obj:`switch.error.SwitchError`: If the group could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.GroupController.get_group`.
        """
        return await self.community_service.groups.get_group(id)

    async def get_all_groups(
        self: "swibots.ApiClient", community_id: str
    ) -> List[Group]:
        """
        Get all groups from the community.
        """
        return await self.community_service.groups.get_all_groups(community_id)

    async def create_group(self: "swibots.ApiClient", group: Group) -> str:
        """Create group

        Args:
            :obj:`swibots.api.community.models.Group`: The group object.

        Returns:
            str:  group id

        Raises:
            :obj:`switch.error.SwitchError`: If the group could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.GroupController.create_group`.
        """
        return await self.community_service.groups.create_group(group)

    async def update_group(self: "swibots.ApiClient", group: Group) -> str:
        """Update group

        Args:
            :obj:`swibots.api.community.models.Group`: The group object.

        Returns:
            str:  group id

        Raises:
            :obj:`switch.error.SwitchError`: If the group could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.GroupController.update_group`.
        """
        return await self.community_service.groups.update_group(group)

    async def delete_group(self: "swibots.ApiClient", id: str) -> bool:
        """Delete group

        Args:
            :obj:`swibots.api.community.models.Group`: The group object.

        Returns:
            bool: Whether the group was deleted!

        Raises:
            :obj:`switch.error.SwitchError`: If the group could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.GroupController.delete_group`.
        """
        return await self.community_service.groups.delete_group(id)



import os, asyncio
from enum import Enum
from io import BytesIO
from typing import Any, Callable, Collection, Coroutine, Dict, TypeVar, Union
from inspect import iscoroutinefunction
from swibots.errors import CancelError
from threading import Thread


class IOClient:
    def __init__(self) -> None:
        self._cancelled = False

    def cancel(self) -> None:
        self._cancelled = True
        raise CancelError("called cancel()")


class DownloadProgress:
    def __init__(
        self, downloaded: int, total: int, url: str, client: IOClient, file_name: str
    ):
        self.downloaded = downloaded
        self.total = total
        self.url = url
        self.client = client
        self.file_name = file_name
        self.started = False


class UploadProgress:
    def __init__(
        self,
        path: str = None,
        callback=None,
        callback_args: tuple = (),
        client: IOClient = None,
        readed: int = 0,
        current: int = 0,
        size: int = 0,
    ) -> None:
        super().__init__()
        self.callback = callback
        _bytesio = isinstance(path, BytesIO)
        self.obj = path
        self.total = size
        if not size:
            self.total = path.getbuffer().nbytes if _bytesio else os.path.getsize(path)
        self.path = path.name if _bytesio else path
        self.callback_args = callback_args
        self.readed, self.current = readed, current
        self.client = client

    def update(self, bytes):
        self.current = bytes
        self.readed += bytes
        self.runCallback(self.current)

    def bytes_completed(self, byte_count):
        if self.readed and byte_count > self.readed:
            self.current = byte_count - self.readed
        else:
            self.current = byte_count
        self.readed = byte_count
        self.runCallback(self.current)

    async def bytes_readed(self, length):
        self.current = length
        self.readed += length
        if self.callback:
            iscoro = self.callback(
                UploadProgress(
                    self.path,
                    self.callback,
                    self.callback_args,
                    self.client,
                    readed=self.readed,
                    current=length,
                    size=self.total,
                ),
                *self.callback_args or (),
            )
            if iscoroutinefunction(self.callback):
                asyncio.create_task(iscoro)

    def runCallback(self, current):
        if self.callback:
            iscoro = self.callback(
                UploadProgress(
                    self.path,
                    self.callback,
                    self.callback_args,
                    self.client,
                    readed=self.readed,
                    current=current,
                ),
                *self.callback_args or (),
            )
            if iscoroutinefunction(self.callback):
                th = Thread(target=lambda: asyncio.run(iscoro))
                th.start()
        return

    def set_total_bytes(self, total):
        self.total = total


CtxType = TypeVar("CtxType")
ResType = TypeVar("ResType")
HandlerCallback = Callable[[CtxType], Coroutine[Any, Any, ResType]]
FilterCallback = Callable[[CtxType], Coroutine[Any, Any, bool]]
DownloadProgressCallback = Callable[[DownloadProgress], Coroutine[Any, Any, None]]
UploadProgressCallback = Callable[[DownloadProgress], Coroutine[Any, Any, None]]

RT = TypeVar("RT")
SCT = Union[RT, Collection[RT]]
"""Single instance or collection of instances."""

JSONDict = Dict[str, Any]


class ReadCallbackStream(object):
    """Wraps a file-like object in another, but also calls a user
    callback with the number of bytes read whenever its `read()` method
    is called. Used for tracking upload progress, for example for a
    progress bar in a UI application. Idea taken from ActiveState Code Recipe:
    http://code.activestate.com/recipes/578669-wrap-a-string-in-a-file-like-object-that-calls-a-u/
    """

    def __init__(self, file_like, callback: Callable = None):
        if isinstance(file_like, str):
            self.file_like = open(file_like, "rb")
        else:
            self.file_like = file_like
        self.callback = callback
        self.cancelled = False

    def read(self, *args):
        if self.cancelled:
            raise CancelError("Task has been cancelled!")

        chunk = self.file_like.read(*args)
        if self.callback and len(chunk) > 0:
            self.callback(len(chunk))
        return chunk

    def close(self):
        if hasattr(self.file_like, "close"):
            self.file_like.close()


class RequestMethod(Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"



import swibots
from typing import Optional
from swibots.base import SwitchObject
from swibots.utils.types import JSONDict


class UserTournament(SwitchObject):
    def __init__(
        self,
        app: Optional["swibots.App"] = None,
        id: Optional[str] = None,
        type: Optional[type] = None,
        name: Optional[str] = None,
        user_id: Optional[str] = None,
        xp_earned: Optional[int] = None,
    ):
        super().__init__(app)
        self.id = id
        self.type = type
        self.name = name
        self.user_id = user_id
        self.xp_earned = xp_earned

    def to_json(self) -> JSONDict:
        return {
            "id": self.id,
            "type": self.type,
            "name": self.name,
            "userId": self.user_id,
            "xpEarned": self.xp_earned,
        }

    @classmethod
    def from_json(self, data: JSONDict = None) -> "UserTournament":
        if data:
            self.id = data.get("id")
            self.type = data.get("type")
            self.name = data.get("name")
            self.user_id = data.get("userId")
            self.xp_earned = data.get("xpEarned")
        return self



import logging
from typing import TYPE_CHECKING
from swibots.api.community.models import BanInfo

if TYPE_CHECKING:
    from swibots.api.community import CommunityClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/community/ban"


class BanController:
    def __init__(self, client: "CommunityClient"):
        self.client = client

    async def ban_user(self, community_id: str, user_id: str):
        """ban a user from community"""
        response = await self.client.post(
            f"{BASE_PATH}", data={"communityId": community_id, "userId": user_id}
        )
        return self.client.build_object(BanInfo, response.data.get("result"))

    async def unban_user(self,  community_id: str,  user_id: str):
        response = await self.client.post(
            f"{BASE_PATH}/unban",
            data={
                "communityId": community_id,
                "userId": user_id,
            },
        )
        return response.data.get("result")



import swibots
from swibots.base import SwitchObject
from logging import getLogger
from swibots.utils.types import JSONDict
from .types import ScreenType, Component, Icon, Text
from typing import List, Optional, Dict, Any, Union
from .BottomBar import BottomBar
from .ListView import ListView

LOG = getLogger(__name__)


class AppBar(SwitchObject):
    def __init__(
        self,
        title: str = "App",
        subtitle: str = "",
        left_icon: Union[Icon, str] = Icon(
            "https://raw.githubusercontent.com/switchcollab/Switch-Bots-Python-Library/main/docs/static/img/logo.png"
        ),
        secondary_icon: Union[Icon, str] = None,
        tertiary_icon: Union[Icon, str] = "",
    ):
        self.title = title
        self.subtitle = subtitle

        if isinstance(left_icon, str):
            left_icon = Icon(left_icon)
        self.left_icon = left_icon

        if isinstance(secondary_icon, str):
            secondary_icon = Icon(secondary_icon)
        self.secondary_icon = secondary_icon

        if isinstance(tertiary_icon, str):
            tertiary_icon = Icon(tertiary_icon)
        self.tertiary_icon = tertiary_icon

    def to_json(self) -> JSONDict:
        data = {"title": self.title, "subtitle": self.subtitle}
        if self.left_icon:
            data["leftIcon"] = self.left_icon.to_json()
        if self.secondary_icon:
            data["secondaryIcon"] = self.secondary_icon.to_json()
        if self.tertiary_icon:
            data["tertiaryIcon"] = self.tertiary_icon.to_json()
        return data


class AppPage(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        screen: ScreenType = ScreenType.SCREEN,
        components: List[Component] = None,
        app_bar: AppBar = None,
        bottom_bar: BottomBar = None,
        show_continue: bool = True,
        back_action: str = None,
        disable_appbar: bool = False,
        max_size: bool = True,
        on_close: str = None,
        **kwargs
    ):
        super().__init__(app)
        self.type = "appPage"
        self.screen = screen
        self.back_action = back_action
        layouts = kwargs.get("layouts")
        self.components = components or []
        if layouts:
            self.components.extend(layouts)
        self.disable_appbar = disable_appbar
        self.app_bar = app_bar
        self.bottom_bar = bottom_bar
        self.show_continue = show_continue
        self.max_size = max_size
        self.on_close = on_close

    def to_json(self) -> JSONDict:
        components = []
        for component in self.components:

            if isinstance(component, ListView):
                if self.max_size != None:
                    component.max_size = self.max_size
                parsed = component.to_json_request()
                if isinstance(parsed, list):
                    components.extend(parsed)
                else:
                    components.append(parsed)
            elif isinstance(component, Component):
                componentJson = component.to_json()
                if self.max_size != None:
                    componentJson["mainAxisSize"] = "max" if self.max_size else "min"
                components.append(componentJson)
            elif isinstance(component, str):
                components.append(Text(component))

        data = {
            "type": self.type,
            "mode": self.screen.value,
            "components": components,
        }
        if not self.disable_appbar and self.app_bar:
            data["pageBar"] = self.app_bar.to_json()
        if self.bottom_bar:
            data.update(self.bottom_bar.to_json())
        if self.show_continue:
            data["showContinue"] = self.show_continue
        if self.back_action:
            data["pageId"] = self.back_action
        if self.on_close:
            data["onClose"] = self.on_close
        return data

    def from_json(self, data: dict) -> Any:
        if data is not None:
            self.screen = ScreenType(data.get("mode", "screen"))
            self.disable_appbar = data.get("disable_appbar", False)
            self.show_continue = data.get("showContinue", True)
            self.back_action = data.get("pageId")
            self.on_close = data.get("onClose")
            self.max_size = data.get("maxSize")
            self.app_bar = AppBar().from_json(data.get("pageBar"))
            self.bottom_bar = BottomBar().from_json(data)
            self.components = [
                Component.build_from_json(item) for item in data.get("components", [])
            ]

        return self



import swibots
from typing import List, Optional

from swibots.api.common.models import User
from swibots.api.community.models import RoleMember


class RoleMemberMethods:
    """
    Methods for managing member linked with role in a community.

    Args:
        client (swibots.ApiClient): The API client.
    """

    async def get_members(self: "swibots.ApiClient", role_id: str) -> List[RoleMember]:
        """
        Get the members of the role with the specified role ID.

        Args:
            role_id: The ID of the role.

        Returns:
            A list of the members of the role.
        """

        return await self.community_service.rolemember.get_members(role_id)

    async def add_member_to_role(
        self: "swibots.ApiClient",
        community_id: str,
        member_id: int,
        role_ids: List[int],
    ) -> bool:
        """
        Add a member to the role with the specified role ID.

        Args:
            community_id: The ID of the community.
            member_id: The ID of the member.
            role_ids: The list of ID of the roles.

        Returns:
            True if the member was added successfully, False otherwise.
        """

        return await self.community_service.rolemember.add_member_to_role(
            community_id, member_id, role_ids
        )

    async def delete_role_member(
        self: "swibots.ApiClient",
        id: Optional[int] = None,
        member_id: Optional[int] = None,
        role_id: Optional[int] = None,
    ) -> bool:
        """
        Delete the role member with the specified ID.

        Args:
            id: The ID of the role member.
            member_id: The ID of the member.
            role_id: The ID of the role.

        Returns:
            True if the role member was deleted successfully, False otherwise.
        """

        return await self.community_service.rolemember.delete_role_member(
            id, member_id, role_id
        )



import logging

logging.basicConfig(level=logging.INFO)

from swibots import (
    BotApp,
    RegisterCommand,
    CommandEvent,
    BotContext,
    InlineMarkup,
    InlineKeyboardButton,
    CallbackQueryEvent,
)


TOKEN = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6OTgyLCJpc19ib3QiOnRydWUsImFjdGl2ZSI6dHJ1ZSwiaWF0IjoxNjg5NDE0NTE3LCJleHAiOjIzMjA1NjY1MTd9.DNlqVeGHmlQIEKfj-H9SF9Hb654rMc48YLLWkc1fJoQ"
app = BotApp(TOKEN).register_command([RegisterCommand("start", "get message")])


@app.on_command("start")
async def an(ctx: BotContext[CommandEvent]):
    await ctx.event.message.respond(
        "Hello",
        inline_markup=InlineMarkup(
            [
                [
                    InlineKeyboardButton("With Alert", callback_data="waler"),
                    InlineKeyboardButton("Without", callback_data="wwal"),
                ],
            ]
        ),
    )


@app.on_callback_query()
async def callback(ctx: BotContext[CallbackQueryEvent]):
    show_alert = ctx.event.callback_data == "waler"
    print(ctx.event.query_id)
    print(
        await ctx.event.answer(
            "This is a long text\n\nDo you agree?\nThis is a secret box!\n\nGood if you know",
            show_alert=show_alert,
        )
    )


app.run()



import swibots
from typing import Optional

from swibots.api.community.models import InstantMessaging


class InstantMessagingMethods:
    async def enable_messages(
        self: "swibots.ApiClient",
        community_id: str,
        group_id: str,
        bot_id: Optional[str] = None,
    ) -> InstantMessaging:
        """Enable messaging in group"""
        return await self.community_service.instant_messaging.enable_messages(
            community_id, group_id, bot_id
        )

    async def disable_messages(
        self: "swibots.ApiClient",
        community_id: str,
        group_id: str,
        bot_id: Optional[str] = None,
    ) -> InstantMessaging:
        """Disable messaging in group"""
        return await self.community_service.instant_messaging.disable_messages(
            community_id, group_id, bot_id
        )

    async def get_messaging_enabled(
        self: "swibots.ApiClient", community_id: str, group_id: str
    ) -> InstantMessaging:
        """Get bot with messaging enable in the community"""
        return await self.community_service.instant_messaging.get_messaging_enabled(
            community_id, group_id
        )



import re
from typing import Optional
from asyncio import create_task, iscoroutine
from swibots.api.chat.events import MessageEvent
from swibots.api.chat.events.chat_event import ChatEvent
from swibots.api.common.events import Event
from swibots.bots.bot_context import BotContext
from swibots.utils.types import SCT, CtxType  # , FilterCallback
from swibots.types import EventType
from typing import Callable, Union, List, Pattern


async def sync_or_async(func):
    if iscoroutine(func):
        return await func
    return func


class Filter:
    callback = None
    value: bool = None
    onFailure = None

    async def __call__(self, ctx: BotContext) -> bool:
        if not self.callback:
            raise NotImplemented
        self.value = await sync_or_async(self.callback(ctx))
        return self.value

    async def check_failure(self, ctx: BotContext):
        if self.onFailure and self.value is False:
            await sync_or_async(self.onFailure(ctx))

    def __invert__(self):
        return InvertFilter(self)

    def __and__(self, other):
        return AndFilter(self, other)

    def __or__(self, other):
        return OrFilter(self, other)


class InvertFilter(Filter):
    def __init__(self, base: Filter):
        self.base = base

    async def __call__(self, ctx: BotContext) -> bool:
        result = self.value = await self.base(ctx)
        if not result:
            await self.base.check_failure(ctx)
        return not result


class AndFilter(Filter):
    def __init__(self, base: Filter, other: Filter):
        self.base = base
        self.other = other

    async def check_failure(self, ctx):
        if self.value is False:
            if self.base.value and not self.other.value:
                await self.other.check_failure(ctx)
            elif self.other.value and not self.base.value:
                await self.base.check_failure(ctx)

    async def __call__(self, ctx: BotContext) -> bool:
        r1 = await self.base(ctx)
        r2 = await self.other(ctx)
        self.value = r1 and r2
        return self.value


class OrFilter(Filter):
    def __init__(self, base, other):
        self.base = base
        self.other = other

    async def check_failure(self, ctx):
        if self.value is False:
            await self.base.check_failure(ctx)
            await self.other.check_failure(ctx)

    async def __call__(self, ctx: BotContext) -> bool:
        r1 = await self.base(ctx)
        # short circuit
        if r1:
            self.value = True
            return True

        result = self.value = await self.other(ctx)
        return result


CUSTOM_FILTER_NAME = "CustomFilter"


class FilterCallback:
    def __init__(self, callback, onSuccess=None, onFallback=None) -> None:
        self.callback = callback
        self.onSuccess = onSuccess
        self.onFallback = onFallback


def create(func: FilterCallback, name: str = None, **kwargs) -> Filter:
    if not isinstance(func, FilterCallback):
        func = FilterCallback(func)
    return type(
        name or func.callback.__name__ or CUSTOM_FILTER_NAME,
        (Filter,),
        {
            "callback": func.callback,
            "onSuccess": func.onSuccess,
            "onFallback": func.onFallback,
            **kwargs,
        },
    )()


async def all_filter(self, ctx: BotContext):
    return True


all = create(all_filter)
"""Filter all messages."""


async def self_filter(self, ctx: BotContext[MessageEvent]):
    return (
        ctx.event.message is not None
        and ctx.event.message.user_id == ctx.event.message.receiver_id
    )


self = create(self_filter)
"""Filter messages generated by the same user."""


async def bot_filter(self, ctx: BotContext[MessageEvent]):
    return (
        ctx.event.message is not None
        and ctx.event.user is not None
        and ctx.event.user.is_bot
    )


is_bot = create(bot_filter)
"""Filter messages coming from bots."""


async def personal_filter(self, ctx: BotContext[MessageEvent]):
    return ctx.event.message.personal_chat


personal = create(personal_filter)


async def communities_filter(self, ctx: BotContext[MessageEvent]):
    return ctx.event.message.community_id


communities = create(communities_filter, name="communities")

channels = create(lambda _, ctx: ctx.event.message.channel_chat)
groups = create(lambda _, ctx: ctx.event.message.group_chat)

video = create(lambda _, ctx: ctx.event.message.video)
photo = create(lambda _, ctx: ctx.event.message.photo)
document = create(lambda _, ctx: ctx.event.message.document)
audio = create(lambda _, ctx: ctx.event.message.audio)
sticker = create(lambda _, ctx: ctx.event.message.sticker)


async def me_filter(self, ctx: BotContext[MessageEvent]):
    return ctx.event.message is not None and ctx.event.message.user_id == ctx.user.id


me = create(me_filter)
"""Filter messages coming from your user."""


async def incoming_filter(self, ctx: BotContext[MessageEvent]):
    return (
        ctx.event.message is not None and ctx.event.message.receiver_id == ctx.user.id
    )


incoming = create(incoming_filter)
"""Filter incoming messages. Messages that are sent to the users recipient."""


async def outgoing_filter(self, ctx: BotContext[MessageEvent]):
    return not incoming(ctx)


outgoing = create(outgoing_filter)
"""Filter outgoing messages. Messages that are sent by the user."""


async def admin_filter(self, ctx: BotContext[MessageEvent]):
    m = ctx.event.message
    if not m.community_id:
        return False
    return await ctx.is_admin(m.community_id, m.user_id)


admin = create(admin_filter)


class community(Filter, set):
    """Filter events comming from a specific community or communities"""

    def __init__(self, community_id: Optional[SCT[str]]):
        self.community_id = community_id

    async def __call__(self, ctx: BotContext[Event]):
        community_id = self.community_id
        if community_id is None:
            return ctx.event.community_id is not None
        if isinstance(community_id, str):
            community_id = frozenset({community_id})
        else:
            community_id = frozenset(community_id)
        return ctx.event.community_id in community_id


class channel(Filter, set):
    """Filter events comming from a specific channel or channels"""

    def __init__(self, channel_id: Optional[SCT[str]]):
        self.channel_id = channel_id

    async def __call__(self, ctx: BotContext[Event]):
        channel_id = self.channel_id
        if channel_id is None:
            return ctx.event.channel_id is not None
        if isinstance(channel_id, str):
            channel_id = frozenset({channel_id})
        else:
            channel_id = frozenset(channel_id)
        return ctx.event.channel_id in channel_id


class group(Filter, set):
    def __init__(self, group_id: Optional[SCT[str]]):
        self.group_id = group_id

    async def __call__(self, ctx: BotContext[Event]):
        group_id = self.group_id
        if group_id is None:
            return ctx.event.group_id is not None
        if isinstance(group_id, str):
            group_id = frozenset({group_id})
        else:
            group_id = frozenset(group_id)
        return ctx.event.group_id in group_id


class user(Filter, set):
    def __init__(self, user_id: Optional[SCT[int]]):
        self.user_id = user_id

    async def __call__(self, ctx: BotContext[Event]):
        user_id = self.user_id
        if user_id is None:
            return ctx.event.action_by_id is not None
        if isinstance(user_id, int):
            user_id = frozenset({user_id})
        else:
            user_id = frozenset(user_id)
        return ctx.event.action_by_id in user_id


def text(text: Optional[SCT[str]]):
    async def func(self, ctx: BotContext[MessageEvent]):
        text = self.text
        if text is None:
            return (
                ctx.event.message is not None and ctx.event.message.message is not None
            )

        if isinstance(text, str):
            text = frozenset({text})
        else:
            text = frozenset(text)

        if ctx.event.type == EventType.MESSAGE:
            value = ctx.event.message.message
        elif ctx.event.type == EventType.COMMAND:
            value = ctx.event.command
        elif ctx.event.type == EventType.CALLBACK_QUERY:
            value = ctx.event.callback_data

        for t in text:
            if t in value:
                return True
            else:
                try:
                    regexp = re.compile(t)
                    if regexp.search(value):
                        return True
                except re.error:
                    pass
        return False

    return create(func, name="TextFilter", text=text)


""" Filter messages by text. """


def regexp(regexp: Optional[SCT[str]]):
    async def func(self, ctx: BotContext[MessageEvent]):
        regexp = self.regexp
        if regexp is None:
            return (
                ctx.event.message is not None and ctx.event.message.message is not None
            )
        if isinstance(regexp, str):
            regexp = frozenset({regexp})
        else:
            regexp = frozenset(regexp)

        if ctx.event.type == EventType.MESSAGE:
            value = ctx.event.message.message
        elif ctx.event.type == EventType.COMMAND:
            value = ctx.event.command
        elif ctx.event.type == EventType.CALLBACK_QUERY:
            value = ctx.event.callback_data

        if value is None:
            return False

        for r in regexp:
            try:
                cr = re.compile(r)
                if cr.match(value):
                    if not isinstance(ctx.event.message.matches, list):
                        ctx.event.message.matches = []
                    ctx.event.message.matches.append(cr.match(value))
                    return True
            except re.error:
                pass
        return False

    return create(func, name="RegexpFilter", regexp=regexp)


""" Filter messages by regexp. """


async def message_type(types: Optional[SCT[int]]):
    async def func(self, ctx: BotContext[MessageEvent]):
        types = self.types
        if types is None:
            return ctx.event.message.status is not None
        if isinstance(types, int):
            types = frozenset({types})
        else:
            types = frozenset(types)
        return ctx.event.message.status in types

    return create(func, name="MessageType", types=types)



from typing import Optional
import swibots
from swibots.api.common.events.event import Event
from swibots.api.community.models.channel import Channel
from swibots.api.community.models.community import Community
from swibots.api.community.models.group import Group
from swibots.api.common.models.user import User
from swibots.api.chat.models.message import Message
from swibots.types import EventType
from swibots.utils.types import JSONDict


class ChatEvent(Event):
    def __init__(
        self,
        app: "swibots.App" = None,
        type: Optional[EventType] = None,
        community_id: Optional[str] = None,
        community: Optional[Community] = None,
        group_id: Optional[str] = None,
        group: Optional[Group] = None,
        channel_id: Optional[str] = None,
        channel: Optional[Channel] = None,
        action_by_id: Optional[int] = None,
        action_by: Optional[User] = None,
        data: Optional[dict] = None,
        user_id: Optional[int] = None,
        user: Optional[User] = None,
    ):
        super().__init__(
            app=app,
            type=type,
            data=data,
            action_by=action_by,
            action_by_id=action_by_id,
            channel=channel,
            channel_id=channel_id,
            group=group,
            group_id=group_id,
            community=community,
            community_id=community_id,
        )
        self.user_id = user_id
        self.user = user

    def to_json(self) -> JSONDict:
        d = super().to_json()

        d.update(
            {
                "userId": self.user_id,
                "user": self.user.to_json(),
            }
        )
        return d

    def from_json(self, data: JSONDict) -> "ChatEvent":
        if data is not None:
            super().from_json(data)
            details = data.get("details") or {}
            self.user_id = int(
                data.get("userId")
                or data.get("senderId")
                or data.get("actionById")
                or 0
            )
            self.user = User.build_from_json(
                details.get("user") or details.get("sender") or data.get("actionBy"),
                self.app,
            )
            self.community_id = data.get("communityId")
            self.community = Community.build_from_json(
                details.get("community"), self.app
            )
            self.group_id = data.get("groupId")
            self.group = Group.build_from_json(details.get("group"), self.app)
            self.channel_id = data.get("channelId")
            self.channel = Channel.build_from_json(details.get("channel"), self.app)
            self.action_by_id = int(data.get("actionById", 0))
            self.action_by = User.build_from_json(data.get("actionBy"), self.app)

        return self



from swibots.utils.types import JSONDict
from .types import Component, Icon
from typing import Optional, List

from enum import Enum


class KeyboardType(Enum):
    NUMBER = "number"
    TEXT = "text"
    EMAIL = "email"
    URL = "url"


class TextInput(Component):
    type = "input_box"

    def __init__(
        self,
        label: str,
        value: str = "",
        width: int = 0,
        placeholder: Optional[str] = "",
        callback_data: Optional[str] = "",
        keyboardType: KeyboardType = KeyboardType.TEXT,
        error: bool = False,
        error_text: Optional[str] = "",
        password: bool = False,
        right_icon: Optional[Icon] = None,
        left_icon: Optional[Icon] = None,
        multiline: bool = False,
        expanded: bool = False,
        max_size: bool = None,
    ):
        self.label = label
        self.width = width
        self.value = value
        self.placeholder = placeholder
        self.callback_data = callback_data
        self.keyboard_type = keyboardType
        self.error = error
        self.error_text = error_text
        self.password = password
        self.multiline = multiline
        self.expanded = expanded
        self.max_size = max_size

        if isinstance(right_icon, str):
            right_icon = Icon(right_icon)
        self.right_icon = right_icon

        if isinstance(left_icon, str):
            left_icon = Icon(left_icon)
        self.left_icon = left_icon

    def to_json(self):
        data = {
            "type": self.type,
            "label": self.label,
            "width": self.width,
            "value": self.value,
            "placeholder": self.placeholder,
            "callbackData": self.callback_data,
            "keyboardType": self.keyboard_type.value,
            "error": self.error,
            "errorText": self.error_text,
            "password": self.password,
        }
        if self.right_icon:
            data["rightIcon"] = self.right_icon.to_json()
        if self.left_icon:
            data["leftIcon"] = self.left_icon.to_json()
        if self.multiline:
            data["expansion"] = "flexible_expansion"
        elif self.expanded:
            data["expansion"] = "expanded_expansion"

        data["mainAxisSize"] = "max" if self.max_size else "min"
        return data


class FilePicker(Component):
    type = "file_picker"

    def __init__(
        self,
        callback_data: str,
        files_count: int = 1,
        mime_type: List[str] = ["png", "jpg", "jpeg", "webp"],
        max_size: bool = None,
    ):
        self.callback_data = callback_data
        self.files_count = files_count
        self.mime_type = mime_type
        self.max_size = max_size

    def to_json(self):
        return {
            "type": self.type,
            "callbackData": self.callback_data,
            "numberOfFiles": self.files_count,
            "mimeType": self.mime_type,
            "mainAxisSize": "max" if self.max_size else "min",
        }



import swibots
from typing import Optional
from swibots.api.community.models import Channel


class DeductXP:
    async def deduct_xp(
        self: "swibots.ApiClient",
        community_id: str,
        user_id: str,
        xp: int = 0,
        description: Optional[str] = None,
    ) -> bool:
        """
        Deducts XP from a user in a community.

        Args:
            community_id: The ID of the community.
            user_id: The ID of the user.
            xp: The amount of XP to deduct.
            description: An optional description of the reason for the deduction.

        Returns:
            True if the deduction was successful, False otherwise.
        """
        return await self.community_service.communities.deduct_xp(
            community_id, user_id, xp, description
        )



import swibots
from swibots.utils.types import JSONDict
from swibots.base import SwitchObject

from .base_typed_inline_query_result import BaseTypedInlineQueryResult
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup
from .types import InlineQueryResultType


class InlineQueryResultVideo(BaseTypedInlineQueryResult):
    def __init__(
        self,
        id: str,
        title: str,
        video_url: str,
        mime_type: str = None,
        video_width: int = None,
        video_height: int = None,
        video_duration: int = None,
        caption: str = None,
        input_message: "InputMessageContent" = None,
        description: str = None,
        thumb_url: str = None,
        thumb_width: int = None,
        thumb_height: int = None,
        reply_markup: "InlineMarkup" = None,
    ):
        super().__init__(
            id,
            InlineQueryResultType.VIDEO,
            title,
            description,
            thumb_url,
            thumb_width,
            thumb_height,
            input_message,
            reply_markup,
        )
        self.video_url = video_url
        self.video_width = video_width
        self.video_height = video_height
        self.video_duration = video_duration
        self.caption = caption
        self.mime_type = mime_type

    def to_json(self) -> JSONDict:
        data = super().to_json()
        data.update(
            {
                "videoUrl": self.video_url,
                "videoWidth": self.video_width,
                "videoHeight": self.video_height,
                "videoDuration": self.video_duration,
                "caption": self.caption,
                "mimeType": self.mime_type,
            }
        )
        return data



class WsMessage:
    def __init__(self, raw):
        self.raw = raw
        self.headers = raw.headers
        self.body = raw.body
        self.command = raw.command
        self.destination = raw.headers.get("destination")
        self.subscription = raw.headers.get("subscription")
        self.message_id = raw.headers.get("message-id")
        self.ack = raw.headers.get("ack")
        self.nack = raw.headers.get("nack")



from swibots.utils.types import JSONDict
from .types import Component
from enum import Enum


class Size(Enum):
    LARGE = "large"
    MEDIUM = "medium"
    SMALL = "small"


class ProgressStyle(Enum):
    LINEAR = "linear"
    CIRCULAR = "circular"


class Progress(Component):
    type = "progress_bar"

    def __init__(
        self,
        color: str = None,
        size: Size = Size.MEDIUM,
        radius: int = 2,
        progress: int = False,
        animation: bool = False,
        progress_style: ProgressStyle = ProgressStyle.LINEAR,
    ):
        self.color = color
        self.size = size
        self.radius = radius
        self.progress = progress
        self.animation = animation
        self.progress_style = progress_style

    def to_json(self):
        return {
            "type": self.type,
            "color": self.color,
            "radius": self.radius,
            "interdeterminate": self.animation,
            "progress": self.progress,
            "barSize": self.size.value,
            "progressBarType": self.progress_style.value,
        }


class ListTileProgress(Progress):
    def __init__(
        self, color: str = None, progress: str = None, animation: bool = False
    ):
        super().__init__(color, progress=progress, animation=animation)



file = "file {r}.mp4"
print(file.format(d="ahh"))
exit()
from PIL import Image, ImageFilter
img = Image.open("y.png")
img.thumbnail((img.height // 2, img.width // 2), Image.BILINEAR)
img.filter(ImageFilter.GaussianBlur(6)).save("x.png", quality=32, optimize=True)


from typing import List
import swibots
from swibots.api.chat.models import Message
from typing import Literal, Union
from swibots.api.community.models import Channel, Group, SearchResultUser


class GetCommunityMediaFiles:
    async def get_community_media_files(
        self: "swibots.ApiClient", community_id: str
    ) -> List[Message]:
        """Get community media files

        Parameters:
            community_id (``int``): The community id

        Returns:
            ``List[~switch.api.chat.models.Message]``: The messages

        Raises:
            ``~switch.error.SwitchError``: If the messages could not be retrieved

        This function does the same as :meth:`~switch.api.chat.controllers.MessageController.get_messages`.
        """
        return await self.chat_service.messages.get_community_media_files(community_id)

    async def search_community_data(
        self: "swibots.ApiClient",
        query: str,
        community_id: str,
        filter: Literal[
            "MESSAGE", "MEDIA", "LINK", "GROUP", "CHANNEL", "MEMBER"
        ] = "MESSAGE",
        limit: int = 10,
        page: int = 0,
    ) -> Union[List[Message], List[Group], List[Channel], List[SearchResultUser]]:
        """Search community data

        Parameters:
            query (``str``): The search query
            community_id (``str``): The community id
            filter (``str``, *optional*): The filter. Defaults to "MESSAGES".
            limit (``int``, *optional*): The limit. Defaults to 10.
            page (``int``, *optional*): The page. Defaults to 0.

        Returns:
            Union[List[Message], List[Group], List[Channel], List[SearchResultUser]]: The search results

        Raises:
            ``~switch.error.SwitchError``: If the search results could not be retrieved
        """
        return await self.chat_service.messages.search_community_data(
            query, community_id, filter, limit, page
        )



from .ad_methods import AdMethods
from .clear_conversation import ClearConversation
from .delete_message import DeleteMessage
from .delete_messages_from_user import DeleteMessagesFromUser
from .edit_message import EditMessage
from .get_user import GetUser
from .flag_message import FlagMessage
from .forward_message import ForwardMessage
from .get_channel_chat_history import GetChannelChatHistory
from .get_group_chat_history import GetGroupChatHistory
from .listen_messages import ListenMessages
from .get_community_media_files import GetCommunityMediaFiles
from .get_community_media_files_by_status import GetCommunityMediaFilesByStatus
from .get_flag_messages import GetFlagMessages
from .flag_message import FlagMessage
from .get_message import GetMessage
from .get_messages_between_users import GetMessagesBetweenUsers
from .get_messages import GetMessages
from .get_unread_messages_count import GetUnreadMessagesCount
from .get_user_media_files import GetUserMediaFiles
from .send_message import SendMessage
from .answer_inline_query import AnswerInlineQuery
from .download_media import DownloadMedia
from .upload_media import UploadMedia
from .sticker_methods import StickerMethods
from .organization_methods import OrganizationMethods
from .heading_methods import HeadingMethods


class ChatMethods(
    AdMethods,
    ClearConversation,
    DeleteMessage,
    DeleteMessagesFromUser,
    EditMessage,
    FlagMessage,
    ForwardMessage,
    GetChannelChatHistory,
    GetGroupChatHistory,
    GetCommunityMediaFiles,
    GetCommunityMediaFilesByStatus,
    GetFlagMessages,
    GetMessage,
    GetMessagesBetweenUsers,
    GetMessages,
    GetUnreadMessagesCount,
    GetUserMediaFiles,
    SendMessage,
    AnswerInlineQuery,
    DownloadMedia,
    ListenMessages,
    UploadMedia,
    GetUser,
    StickerMethods,
    OrganizationMethods,
    HeadingMethods,
):
    pass



import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from swibots.api.chat import ChatClient

from swibots.api.chat.models import ADInfo
from swibots.api.common import User
from urllib.parse import urlencode

log = logging.getLogger(__name__)

BASE_PATH = "/add"


class AdvertisingController:
    """Ads controller"""

    def __init__(self, client: "ChatClient") -> None:
        self.client = client

    async def get_all_ads(
        self,
        limit: int = 100,
        page: int = 0,
    ):
        param = urlencode({"page": page, "limit": limit, "appId": self.client.user.id})
        response = await self.client.get(f"{BASE_PATH}/byAppId?{param}")
        return self.client.build_list(ADInfo, response.data)

    async def get_organization_followers(self, id: str):
        response = await self.client.get(f"{BASE_PATH}/followers?orgId={id}")
        return self.client.build_list(User, response.data)



from swibots import (
    BotApp,
    RegisterCommand,
    BotContext,
    CallbackQueryEvent,
    CommandEvent,
    InlineMarkup,
    InlineKeyboardButton,
    regexp,
    CommunityUpdatedEvent,
    MessageEvent,
    UploadProgress,
    Message,
    DownloadProgress,
    MediaUploadRequest,
)
import logging

logging.basicConfig(level=logging.INFO)

TOKEN = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6OTc1LCJpc19ib3QiOnRydWUsImFjdGl2ZSI6dHJ1ZSwiaWF0IjoxNjg5MTU5NjEzLCJleHAiOjIzMjAzMTE2MTN9.lfLspX1PKUdOdKDPMFHe3PtGSycdUEnA95QbUahSB6k"
 #"eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6OTgyLCJpc19ib3QiOnRydWUsImFjdGl2ZSI6dHJ1ZSwiaWF0IjoxNjg5NDE0NTE3LCJleHAiOjIzMjA1NjY1MTd9.DNlqVeGHmlQIEKfj-H9SF9Hb654rMc48YLLWkc1fJoQ"
# "YOUR_TOKEN_HERE"
MSG = {}
app = BotApp(
    TOKEN
)
import json

async def main():
    await app.start()
#    com = await app.get_community(username="fantasyverse")
    print(
        await app.get_community_members("e7673c9d-926d-46fb-9039-dbea9e3a4b07")
    )
    return
    for _ in range(765548, 768000):
        try:
            msg = await app.get_message(_)
            MSG[msg.id] = {
                "text": msg.message,
                "chat_name": msg.community or msg.user,
                "community": msg.community_id,
                "user": msg.receiver_id
            }
            print(_, msg.message)
        except Exception as er:
            print(er)
            print(_)

import asyncio
try:
    pass
#    asyncio.run(main())
except KeyboardInterrupt:
    pass
with open("save.json", "w") as f:
    json.dump(MSG, f)

# exit()
app = BotApp(
    TOKEN, "A cool bot with annotations and everything you could possibly want :)"
).register_command(
    [
        RegisterCommand("echo", "Echoes the message", True),
        RegisterCommand("buttons", "Shows buttons", True),
        RegisterCommand("buttonfull", "Shows buttons", True),
        RegisterCommand("back", "Shows buttons", True),
        RegisterCommand("upload", "Reply with media", True),
    ]
)


@app.on_command("buttons")
async def buttons_handler(ctx: BotContext[CommandEvent]):
    m = await ctx.bot.prepare_response_message(ctx.event.message)
    m.message = f"Please select an option:"

    inline_keyboard = [
        [
            InlineKeyboardButton(text="Option 18", callback_data="option18"),
            InlineKeyboardButton(text="Option 19", callback_data="option19"),
            InlineKeyboardButton(text="Option 20", callback_data="option20"),
        ],
        [
            InlineKeyboardButton(text="Option 33", callback_data="option33"),
            InlineKeyboardButton(text="Option 43", callback_data="option43"),
            InlineKeyboardButton(text="Option 63", callback_data="option63"),
            InlineKeyboardButton(text="Option 73", callback_data="option73"),
        ],
        [
            InlineKeyboardButton(text="Optioniablek", callback_data="option53"),
            InlineKeyboardButton(text="Unitedstates", callback_data="option03"),
        ],
        [
            InlineKeyboardButton(text="Go Back on Press", callback_data="option543"),
        ],
        [
            InlineKeyboardButton(
                text="Go Back on Press for more Movies haha", callback_data="option543"
            ),
        ],
    ]

    m.inline_markup = InlineMarkup(
        inline_keyboard=inline_keyboard,
    )
    await ctx.bot.send_message(m)


@app.on_command("test")
async def test_handler(ctx: BotContext[CommandEvent]):
    m = await ctx.prepare_response_message(ctx.event.message)
    m.message = "Test command received"
    await ctx.send_message(m)


@app.on_command("buttonfull")
async def buttons_handler(ctx: BotContext[CommandEvent]):
    m = await ctx.bot.prepare_response_message(ctx.event.message)
    m.message = f"Please select an option:"

    inline_keyboard1 = [
        [
            InlineKeyboardButton(text="Option 1111", callback_data="option1"),
            InlineKeyboardButton(text="Option 1112", callback_data="option2"),
            InlineKeyboardButton(text="Option 1115", callback_data="option5"),
        ],
        [
            InlineKeyboardButton(text="Option 18", callback_data="option18"),
            InlineKeyboardButton(text="Option 19", callback_data="option19"),
            InlineKeyboardButton(text="Option 20", callback_data="option20"),
        ],
        [
            InlineKeyboardButton(text="Optioniablek", callback_data="option53"),
            InlineKeyboardButton(text="Unitedstates", callback_data="option03"),
        ],
        [
            InlineKeyboardButton(text="Go Back on Press", callback_data="option543"),
        ],
        [
            InlineKeyboardButton(
                text="Go Back on Press for more Movies haha", callback_data="option543"
            ),
        ],
    ]

    m.inline_markup = InlineMarkup(
        inline_keyboard=inline_keyboard1,
    )
    await ctx.bot.send_message(m)


@app.on_command("echo")
async def buttons_handler(ctx: BotContext[CommandEvent]):
    m = await ctx.bot.prepare_response_message(ctx.event.message)
    m.message = f"Please select an option:"

    inline_keyboard1 = [
        [
            InlineKeyboardButton(text="Option 1", callback_data="option1"),
            InlineKeyboardButton(text="Option 2", callback_data="option2"),
        ],
        [
            InlineKeyboardButton(text="Option 3", callback_data="option3"),
            InlineKeyboardButton(text="Option 4", callback_data="option4"),
        ],
    ]

    m.inline_markup = InlineMarkup(
        inline_keyboard=inline_keyboard1,
    )
    await m.reply_text(
        "Please select an option (echo):",
        InlineMarkup(
            inline_keyboard=inline_keyboard1,
        ),
    )


@app.on_callback_query()
async def query_callback_handler(ctx: BotContext[CallbackQueryEvent]):
    m = ctx.event.message
    m.message = f"Thank you! I received your callback: {ctx.event.callback_data}"
    m.inline_markup = None
    await ctx.edit_message(m)


@app.on_message()
async def message_handler(ctx: BotContext[MessageEvent]):
    # m = await ctx.prepare_response_message(ctx.event.message)
    # m.message = f"Thank you! I received your message: {ctx.event.message.message}"
    # await ctx.send_message(m)
    print(f"Downloading received message: {ctx.event.message.id}")
    message: Message = ctx.event.message
    if message.media_link is not None:
        print(message.media_link)
        await message.download(
            in_memory=False, block=False, progress=handle_download_progress
        )


async def handle_download_progress(progress: DownloadProgress):
    print(f"Downloaded {progress.downloaded} of {progress.total}")


@app.on_community_update()
async def community_update_handler(ctx: BotContext[CommunityUpdatedEvent]):
    print(ctx.event.community_id + " was updated")


# app.run()


def show_upload_progress(obj: UploadProgress):
    print(f"Uploaded {obj.current} of {obj.readed}")
    if obj == 0:
        return


@app.on_command("upload")
async def upload_handler(ctx: BotContext[CommandEvent]):
    params = ctx.event.params
    media = MediaUploadRequest(
        path=params,
        callback=show_upload_progress,
    )

    r = await ctx.event.message.reply_text(f"Here is your file {params}", media=media)
    print(r)


@app.on_message()
async def check(ctx):
    print(ctx.event.message.message)


app.run()



from typing import TYPE_CHECKING, Optional, TypeVar
from swibots.api.community.events import ChannelUpdatedEvent
from swibots.bots.filters.filter import Filter

from swibots.bots.handlers import BaseHandler
from swibots.bots import BotContext
from .event_handler import EventHandler
from swibots.types import EventType
from swibots.utils.types import HandlerCallback

if TYPE_CHECKING:
    pass

ResType = TypeVar("ResType")


class ChannelUpdatedHandler(EventHandler):
    def __init__(
        self,
        callback: HandlerCallback[BotContext[ChannelUpdatedEvent], ResType],
        filter: Optional[Filter] = None,
        **kwargs,
    ):
        super().__init__(EventType.COMMUNITY_CHANNEL_UPDATE, callback, filter, **kwargs)



from typing import TYPE_CHECKING, Optional
from io import BytesIO
import swibots
from swibots.types import MediaType
from swibots.api.chat.models import Message, InlineMarkup, Sticker
from swibots.api.common.models import EmbeddedMedia

if TYPE_CHECKING:
    from swibots.api import ApiClient


class SendMessage:
    """Methods related to sending messages"""

    async def send_message(
        self: "ApiClient",
        message: str,
        community_id: str = None,
        channel_id: str = None,
        group_id: str = None,
        user_id: Optional[int] = None,
        user_session_id: Optional[str] = None,
        document: Optional[str] = None,
        caption: Optional[str] = None,
        description: Optional[str] = None,
        embed_message: Optional[EmbeddedMedia] = None,
        inline_markup: Optional[InlineMarkup] = None,
        reply_to_message_id: Optional[int] = None,
        **kwargs,
    ) -> Message:
        """Send a message

        Parameters:
            `message`: text to send as message
            `community_id`: Community ID
            `group_id`: Group id
            `channel_id`: Channel ID
            `user_id`: User id to send message
            `user_session_id`: Session ID, present if bot is added as channel in the community.
            ``

        Returns:
            ``~switch.api.chat.models.Message``: The message

        Raises:
            ``~switch.error.SwitchError``: If the message could not be sent
            ``~ValueError``: If Chat I
        """
        return await self.chat_service.messages.send_message(
            message,
            community_id=community_id,
            group_id=group_id,
            channel_id=channel_id,
            user_id=user_id,
            user_session_id=user_session_id,
            document=document,
            description=description,
            caption=caption,
            embed_message=embed_message,
            inline_markup=inline_markup,
            reply_to_message_id=reply_to_message_id,
            **kwargs,
        )

    async def send_sticker(
        self: "ApiClient",
        sticker: Sticker,
        community_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
        user_id: Optional[int] = None,
        user_session_id: Optional[str] = None,
        **kwargs,
    ):
        """Send sticker

        Args:
            `sticker`: Sticker
            `community_id`: Community ID
            `group_id`: Group id
            `channel_id`: Channel ID
            `user_id`: User id to send message
            `user_session_id`: Session ID, present if bot is added as channel in the community.
        """
        if sticker:
            kwargs["media_info"] = sticker.sticker_info
            kwargs["media_link"] = sticker.sticker_info.url
            kwargs["media_info"].id = 0
            kwargs["sticker_pack_id"] = sticker.sticker_pack_id

        return await self.chat_service.messages.send_message(
            message=kwargs.get("message", None),
            community_id=community_id,
            channel_id=channel_id,
            group_id=group_id,
            user_id=user_id,
            user_session_id=user_session_id,
            **kwargs,
        )

    async def send_audio(
        self: "ApiClient",
        audio: str | BytesIO,
        caption: str = "",
        community_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
        user_id: Optional[int] = None,
        user_session_id: Optional[str] = None,
        **kwargs,
    ):
        """Send audio

        Args:
            `audio` `(str | BytesIO)`: audio file path:
            `caption` (str, optional): message caption. Defaults to "".
            `community_id`: Community ID
            `group_id`: Group id
            `channel_id`: Channel ID
            `user_id`: User id to send message
            `user_session_id`: Session ID, present if bot is added as channel in the community.

        Returns:
            Message | Task
        """
        return await self.send_media(
            document=audio,
            user_id=user_id,
            user_session_id=user_session_id,
            community_id=community_id,
            channel_id=channel_id,
            group_id=group_id,
            message=caption,
            media_type=MediaType.AUDIO.value,
            **kwargs,
        )



from typing import Union, List

from swibots.utils.types import JSONDict
from .types import Image, Component, Icon, Text
from enum import Enum


class CardSize(Enum):
    LARGE = "large"
    MEDIUM = "medium"
    SMALL = "small"


class CardStyle(Enum):
    TINT = "tint"
    SHADOW = "shadow"
    OUTLINE = "outline"


class Card(Component):
    type = "card"

    def __init__(
        self,
        size: CardSize = CardSize.MEDIUM,
        title: str = "",
        cardType: str = "",
        subtitle: str = "",
        subtitle2: str = "",
        subtitle3: str = "",
        subtitle4: str = "",
        subtitle5: str = "",
        subtitle6: str = "",
        subtitle7: str = "",
        subtitle8: str = "",
        icon: Icon = "",
        icon2: Icon = "",
        icon3: Icon = "",
        icon4: Icon = "",
        icon5: Icon = "",
        callback_data: str = "",
        #        iconAlignment: str = "",
    ):
        self.title = title
        self.cardType = cardType
        self.subTitle = subtitle
        self.subTitle2 = subtitle2
        self.subTitle3 = subtitle3
        self.subTitle4 = subtitle4
        self.subTitle5 = subtitle5
        self.subTitle6 = subtitle6
        self.subTitle7 = subtitle7
        self.subTitle8 = subtitle8
        #        self.image = image
        if isinstance(icon, str):
            icon = Icon(icon)
        self.icon = icon
        if isinstance(icon2, str):
            icon2 = Icon(icon2)
        self.icon2 = icon2
        if isinstance(icon3, str):
            icon3 = Icon(icon3)
        self.icon3 = icon3
        if isinstance(icon4, str):
            icon4 = Icon(icon4)
        self.icon4 = icon4
        if isinstance(icon5, str):
            icon5 = Icon(icon5)
        self.icon5 = icon5
        self.callback_data = callback_data

    #        self.iconAlignment = iconAlignment
    #        self.size = size

    def to_json(self):
        return {
            "type": self.type,
            "title": self.title,
            "cardType": self.cardType,
            "subTitle": self.subTitle,
            "subTitle2": self.subTitle2,
            "subTitle3": self.subTitle3,
            "subTitle4": self.subTitle4,
            "subTitle5": self.subTitle5,
            "subTitle6": self.subTitle6,
            "subTitle7": self.subTitle7,
            "subTitle8": self.subTitle8,
            #            "image": self.image.to_json() if self.image else None,
            "icon": self.icon.to_json() if self.icon else None,
            "icon2": self.icon2.to_json() if self.icon2 else None,
            "icon3": self.icon3.to_json() if self.icon3 else None,
            "icon4": self.icon4.to_json() if self.icon4 else None,
            "icon5": self.icon5.to_json() if self.icon5 else None,
            "callbackData": self.callback_data,
            #            "size": self.size.value,
        }


class CardView(Component):
    type = "card_view"

    def __init__(
        self,
        cards: List[Card],
        card_size: CardSize = CardSize.LARGE,
        card_style: CardStyle = CardStyle.TINT,
        vertical_size: CardSize = CardSize.MEDIUM,
        scrollable: bool = False,
        horizontal: bool = False,
    ):
        self.card_size = card_size
        self.card_style = card_style
        self.vertical_size = vertical_size
        self.scrollable = scrollable
        self.cards = cards
        self.horizontal = horizontal

    def to_json(self):
        return {
            "type": self.type,
            "cardSize": self.card_size.value,
            "cardStyle": self.card_style.value,
            "verticalSize": self.vertical_size.value,
            "scrollable": self.scrollable,
            "horizontal": self.horizontal,
            "children": [child.to_json() for child in self.cards],
        }


class Banner(Component):
    type = "banner"

    def __init__(
        self,
        title: Union[str, Text],
        heading: Union[str, Text] = "",
        subtitle: Union[str, Text] = "",
        icon: Union[Icon, str] = "",
        background: str = "",
        max_size: bool = None,
    ):
        if isinstance(title, str):
            title = Text(title)
        self.title = title

        if isinstance(heading, str):
            heading = Text(heading)
        self.heading = heading

        if isinstance(subtitle, str):
            subtitle = Text(subtitle)
        self.subtitle = subtitle

        if isinstance(icon, str):
            icon = Icon(icon)
        self.icon = icon
        self.background = background
        self.max_size = max_size

    def to_json(self):
        return {
            "type": self.type,
            "icon": self.icon.to_json() if self.icon else None,
            "heading": self.heading.to_json() if self.heading else None,
            "title": self.title.to_json() if self.title else None,
            "subtitle": self.subtitle.to_json() if self.subtitle else None,
            "background": self.background,
            "mainAxisSize": "max" if self.max_size else "min",
        }



import logging

logging.basicConfig(level=logging.INFO)

from anitopy import parse
from swibots import (
    BotApp,
    RegisterCommand,
    CommandEvent,
    BotContext,
    DownloadProgress,
    UploadProgress,
)
from redis import Redis

app = BotApp(
    "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6MTIwNCwiaXNfYm90Ijp0cnVlLCJhY3RpdmUiOnRydWUsImlhdCI6MTY5MzEzNzczMSwiZXhwIjoyMzI0Mjg5NzMxfQ.xise4b6GtcssFDJZvLDrAF_uMqY1W7RgxidkDYIfpuw"
)

db = Redis(
    "redis-10791.c285.us-west-2-2.ec2.cloud.redislabs.com",
    port=10791,
    password="NBvAqmWbyHdBLzfd9zx7MbKi3xTsM42O",
    encoding="utf-8"
)
if db.ping():
    print("Started redis!")

app.register_command(
    [
        RegisterCommand("start", "Start the bot.", True),
        RegisterCommand("rename", "rename a file", True),
        RegisterCommand("setformat", "set file format", True),
        RegisterCommand("getformat", "Get file format", True)
    ]
)


@app.on_command("start")
async def getStart(ctx: BotContext[CommandEvent]):
    event = ctx.event.message
    await event.reply_text(
        """Hi, I am a file renamer bot!\nSend me a file and reply `/rename filename.ext` to it!
**Example of Auto Renaming:**
Before:
file Name: `[Anime Time] One Piece - 0213 - Round 3! The Round-and-Round Roller Race!.mkv`\

Formats:
'file_extension': 'mkv'
'episode_number': '0213'
'anime_title': 'One Piece'
'release_group': 'Anime Time'
'episode_title': 'Round 3! The Round-and-Round Roller Race!'

**Setting format**
/setformat [ChannelName] {episode_name} ({release_group}).{file_extension}
"""
    )


def get_progress_bar_string(pct):
    p = min(max(pct, 0), 100)
    cFull = int(p // 8)
    p_str = "■" * cFull
    p_str += "□" * (12 - cFull)
    return f"[{p_str}]"


def humanbytes(size):
    if not size:
        return "0 B"
    for unit in ["", "K", "M", "G", "T"]:
        if size < 1024:
            break
        size /= 1024
    if isinstance(size, int):
        size = f"{size}{unit}B"
    elif isinstance(size, float):
        size = f"{size:.2f}{unit}B"
    return size

@app.on_command("setformat")
async def setFileFormat(ctx: BotContext[CommandEvent]):
    event = ctx.event.message
    if not ctx.event.params:
        return await event.reply_text("Provide a format!\nRead /start for detailed info.")
    db.set(event.user_id, ctx.event.params)
    await event.reply_text("Updated!")

@app.on_command("getformat")
async def getFormat(ctx: BotContext[CommandEvent]):
    event = ctx.event.message
    getFormat = db.get(event.user_id)
    if not getFormat:
        return await event.reply_text("Format is not set!")
    await event.reply_text(f"Format:\n{getFormat.decode()}")

@app.on_command("rename")
async def renameBot(ctx: BotContext[CommandEvent]):
    event = ctx.event.message

    if not ctx.event.params:
        return await event.reply_text("Provide a file name")
    replied = await event.get_replied_message()
    if not replied or not replied.is_media:
        return await event.reply_text("Reply to a media message!")
    await processFile(event, replied, ctx.event.params)


async def processFile(event, replied, command):
    msg = await event.reply_text("Starting Process!")

    async def downloadCallback(clb: DownloadProgress):
        try:
            progress = (clb.downloaded / clb.total) * 100
        except ZeroDivisionError:
            progress = 0
        message = f"""
FileName: {clb.file_name}
{get_progress_bar_string(progress)} {int(progress)}
Size: {humanbytes(clb.total)}
"""
        await msg.edit_text(message)

    file = await replied.download(progress=downloadCallback)

    async def uploadCallback(clb: UploadProgress):
        try:
            progress = (clb.readed / clb.total) * 100
        except ZeroDivisionError:
            progress = 0
        message = f"""
Uploading {clb.file_name}
{get_progress_bar_string(progress)} {int(progress)}
Size: {humanbytes(clb.total)}r
"""
        await msg.edit_text(message)

    await event.reply_media(
        document=file,
        message="",
        description=command,
        progress=uploadCallback
    )
    await msg.delete()

@app.on_message()
async def onMessage(ctx: BotContext[CommandEvent]):
    event = ctx.event.message
    print(event)
    if event.media_link and event.is_media:
        print(event.media_link)
        nameFormat = db.get(event.user_id)
        print(nameFormat)
        if not nameFormat:
            return
        name = event.media_info.description
        parsed = parse(name)
        print(parsed)
        if not parsed:
            return
        try:
            new_name = nameFormat.decode().format(parsed)
        except Exception as er:
            return await event.reply_text(str(er))
        await processFile(event, event, new_name)

app.run()



from typing import Union, List

from swibots.utils.types import JSONDict
from .types import Component, Icon


class Accordian(Component):
    def __init__(
        self,
        title: str,
        icon: Union[Icon, str] = "",
        components: List[Component] = None,
    ):
        self.title = title
        if isinstance(icon, str):
            icon = Icon(icon)
        self.icon = icon
        self.components = components

    def to_json(self):
        data = {
            "title": self.title,
            "icon": self.icon.to_json() if self.icon else None,
        }
        if self.components:
            data["components"] = (
                [component.to_json() for component in self.components],
            )
        return data



import asyncio
import uuid
from typing import List

from swibots.errors import SwitchError

from .async_ws_subscription import AsyncWsSubscription
from swibots.utils.ws.common import WsFrame
from websockets.client import connect
from websockets import exceptions
import logging

VERSIONS = "1.1,1.0"

log = logging.getLogger(__name__)


class AsyncWsClient:
    def __init__(
        self,
        url: str,
    ):
        self.url = url
        self.ws = None
        self._loop = asyncio.get_event_loop()
        self.opened = False
        self.connected = False
        self.counter = 0
        self.tasks: List[asyncio.Task] = []
        self._heartbeatTask = None
        self.subscriptions: dict[str, AsyncWsSubscription] = {}
        self._connect_args = {}
        self._connect_callback = None
        self.error_callback = None
        self._connectIntents = 0
        self._connectInterval = 7
        self._incrementalDelay = 3
        self._maxConnectIntents = 50
        self._connecting = False
        self._gracefully_disconnect = False

    async def _start_heartbeat(self):
        elapsed = 0
        while self.connected:
            await asyncio.sleep(0.1)
            elapsed = elapsed + 0.1
            if elapsed >= 5:
                await self.transmit("\n", {})
                elapsed = 0
        log.debug("Heartbeat stopped")

    async def _on_open(self, ws_app, *args):
        self.opened = True

    async def _on_close(self, ws_app, *args):
        self.connected = False
        if self._gracefully_disconnect:
            return
        log.error("Whoops! Lost connection to " + self.url)
        await self._clean_up()
        if self._connectIntents >= self._maxConnectIntents > 0:
            log.error("Max connection attempts reached. Aborting.")
            raise SwitchError("Max connection attempts reached. Aborting.")
        wait_time = self._connectInterval
        if self._connectIntents:
            wait_time += self._connectIntents * self._incrementalDelay
        log.info(f"Waiting for {wait_time}s before retry!")
        await asyncio.sleep(wait_time)
        self._connectIntents += 1
        await self.connect(**self._connect_args)

    async def _on_error(self, ws_app, error, *args):
        await self._clean_up()
        await self._on_close(ws_app, *args)
        if error:
            log.error(error)

    async def _on_message(self, ws_app, message, *args):
        frame = WsFrame.unmarshall_single(message)

        if frame.command != "PONG":
            log.debug("\n<<< " + str(message))
        else:
            log.debug("\n<<< " + frame.command)

        _results = []
        if frame.command == "CONNECTED":
            if self._connectIntents:
                log.info("connected to server " + self.url)
            self.connected = True
            self._connecting = False
            self._connectIntents = 0
            self._heartbeatTask = self._loop.create_task(self._start_heartbeat())
            # resubscribe
            for sub in self.subscriptions.values():
                await self._start_subscription(sub)

            # if self._connect_callback is not None:
            #     _results.append(await self._send_heartbeat(frame))
        elif frame.command == "MESSAGE":
            subscription = frame.headers["subscription"]

            if subscription in self.subscriptions:
                sub = self.subscriptions[subscription]
                message_id = frame.headers["message-id"]

                async def ack(headers):
                    if headers is None:
                        headers = {}
                    return await self.ack(message_id, subscription, headers)

                async def nack(headers):
                    if headers is None:
                        headers = {}
                    return await self.nack(message_id, subscription, headers)

                frame.ack = ack
                frame.nack = nack

                _results.append(self._loop.create_task(sub.receive(frame)))
            else:
                info = "Unhandled received MESSAGE: " + str(frame)
                log.debug(info)
                _results.append(info)
        elif frame.command == "ERROR":
            if self.error_callback is not None:
                _results.append(self.error_callback(frame))
        elif frame.command not in ["PONG", "RECEIPT"]:
            info = "Unhandled received MESSAGE: " + frame.command
            log.debug(info)
            _results.append(info)

        return _results

    async def transmit(self, command, headers, body=None):
        try:
            if self.ws is None:
                return
            out = WsFrame.marshall(command, headers, body)
            if command == "\n":
                out = command
            log.debug("\n>>> {}".format("PING" if command == "\n" else out))
            await self.ws.send(out)
        except exceptions.WebSocketException as er:
            log.error(er)
            await self._on_close(self.ws)
        except Exception as e:
            await self._on_error(self.ws, e)

    async def connect(
        self,
        login=None,
        passcode=None,
        headers=None,
        connect_callback=None,
        error_callback=None,
        timeout=0,
        **kwargs,
    ):
        headers = self._set_default_headers(headers)
        await self._connect(
            login,
            passcode,
            headers,
            connect_callback,
            error_callback,
            timeout,
            **kwargs,
        )

    async def _connect(
        self,
        login=None,
        passcode=None,
        headers=None,
        connect_callback=None,
        error_callback=None,
        timeout=30,
        **kwargs,
    ):
        if self.connected:
            log.debug("Already connected to " + self.url)
            return

        if self._connecting:
            log.debug("Already connecting to " + self.url)
            return

        try:
            self._connecting = True
            self._connect_args = kwargs
            log.debug("Opening web socket...")
            self.ws = await connect(self.url)
            log.debug("Web socket opened.")
            self._loop.create_task(self.read_messages())
            headers = headers if headers is not None else {}
            # headers['host'] = self.url
            headers["accept-version"] = VERSIONS
            headers["heart-beat"] = "10000,10000"
            if login is not None:
                headers["login"] = login
            if passcode is not None:
                headers["passcode"] = passcode
            self._connect_callback = connect_callback
            self.error_callback = error_callback
            await self.transmit("CONNECT", headers)
            # elapsed time
            elapsed = 0
            while not self.connected:
                await asyncio.sleep(1)
                elapsed += 1
                if 0 < timeout < elapsed:
                    raise Exception("Connection timeout")
        except (exceptions.WebSocketException,):
            await self._on_close(self.ws)
        except Exception as e:
            await self._on_error(self.ws, e)

    async def read_messages(self):
        try:
            async for message in self.ws:
                await self._on_message(self.ws, message)
            await self._on_close(self.ws)
        except exceptions.WebSocketException as er:
            log.error(er)
            await self._on_close(self.ws)
        except Exception as e:
            await self._on_error(self.ws, e)

    async def disconnect(self, disconnect_callback=None, headers=None):
        headers = self._set_default_headers(headers)
        await self.transmit("DISCONNECT", headers)
        self._gracefully_disconnect = True
        await self.ws.close()
        await self._clean_up()
        if disconnect_callback is not None:
            disconnect_callback()

    async def _clean_up(self):
        try:
            self.connected = False
            self._connecting = False
            self.opened = False
            self.tasks = []
        except Exception as e:
            log.debug("Error cleaning up: " + str(e))

    async def send(self, destination, headers=None, body=None):
        headers = self._set_default_headers(headers)
        if body is None:
            body = ""
        headers["destination"] = destination
        return await self.transmit("SEND", headers, body)

    async def subscribe(self, destination, callback=None, headers=None):
        headers = self._set_default_headers(headers)
        sub_id = f"sub-{uuid.uuid4()}"
        headers["id"] = sub_id

        sub = AsyncWsSubscription(
            client=self,
            destination=destination,
            callback=callback,
            headers=headers,
            sub_id=sub_id,
        )
        await self._start_subscription(sub)
        self.subscriptions[sub_id] = sub
        return sub

    def _set_default_headers(self, headers):
        return headers or {}

    @staticmethod
    async def _start_subscription(subscription: AsyncWsSubscription):
        return await subscription.start()

    async def unsubscribe(self, sub_id):
        del self.subscriptions[sub_id]
        return await self.transmit("UNSUBSCRIBE", {"id": sub_id})

    async def ack(self, message_id, subscription, headers):
        headers = self._set_default_headers(headers)
        headers["message-id"] = message_id
        headers["subscription"] = subscription
        return await self.transmit("ACK", headers)

    async def nack(self, message_id, subscription, headers):
        headers = self._set_default_headers(headers)
        headers["message-id"] = message_id
        headers["subscription"] = subscription
        return await self.transmit("NACK", headers)



import logging
from typing import TYPE_CHECKING, List, Optional

from swibots.api.chat.models import Message, SessionInfo

if TYPE_CHECKING:
    from swibots.api.chat import ChatClient

log = logging.getLogger(__name__)

BASE_PATH = "/v1/message/post"
CHAT_BASE_PATH = "/chat/v1/"


class ChatController:
    """Chat controller"""

    def __init__(self, client: "ChatClient"):
        self.client = client

    async def pin_message(
        self,
        message: Message | int,
        detail: Optional[str] = None,
        message_type: Optional[str] = "MESSAGE",
        status: Optional[int] = None,
    ) -> bool:
        data = {"status": status, "messageType": message_type, "pinDetails": detail}
        if isinstance(message, Message):
            data = {
                "groupId": message.group_id,
                "communityId": message.community_id,
                "channelId": message.channel_id,
                "userId": message.user_id,
                "messageId": message.id,
            }
        else:
            data["messageId"] = message
        await self.client.post(
            f"{BASE_PATH}/pin",
            data=data,
        )
        return True

    async def get_last_seen(self, user_id: int) -> int:
        response = await self.client.get(f"{CHAT_BASE_PATH}lastseen/{user_id}")
        return response.data.get("lastSeen")

    async def get_session_info(self, session_id: str):
        response = await self.client.get(f"/v1/app/session/get?sessionId={session_id}")
        return self.client.build_object(SessionInfo, response.data)



from .message_controller import MessageController
from .chat_controller import ChatController
from .media_controller import MediaController
from .sticker_controller import StickerController
from .organization_controller import OrganizationController
from .ads_controller import AdvertisingController
from .heading_controller import HeadingsController

__all__ = [
    "AdvertisingController",
    "MessageController",
    "ChatController",
    "MediaController",
    "StickerController",
    "OrganizationController",
    "HeadingsController",
]



import swibots
from typing import Optional, List, Any
from swibots.utils.types import JSONDict
from swibots.base.switch_object import SwitchObject
from .usertournament import UserTournament
from enum import Enum


class UserStatus(Enum):
    OFFLINE = "OFFLINE"
    ONLINE = "ONLINE"


class BotPrivacy(Enum):
    PUBLIC = "public"
    PRIVATE = "private"


class User(SwitchObject):
    def __init__(
        self,
        app: "swibots.App" = None,
        id: Optional[int] = None,
        name: Optional[str] = None,
        bio: Optional[str] = None,
        username: Optional[str] = None,
        image_url: Optional[str] = None,
        active: Optional[bool] = None,
        cover_image: Optional[str] = None,
        deleted: Optional[bool] = None,
        role_info: Optional[str] = None,
        link: Optional[str] = None,
        admin: Optional[bool] = None,
        is_bot: Optional[bool] = None,
        is_app: Optional[bool] = None,
        app_callback: Optional[str] = None,
        is_game: Optional[bool] = None,
        is_friend: Optional[bool] = None,
        tournaments: Optional[List[Any]] = None,
        status: Optional[str] = None,
        privacy: Optional[BotPrivacy] = BotPrivacy.PUBLIC,
    ):
        super().__init__(app)
        self.id = id
        self.name = name
        self.bio = bio
        self.username = username
        self.image_url = image_url
        self.active = active
        self.deleted = deleted
        self.link = link
        self.is_app = is_app
        self.app_callback = app_callback
        self.role_info = role_info
        self.admin = admin
        self.cover_image = cover_image
        self.is_bot = is_bot
        self.is_friend = is_friend
        self.is_game = is_game
        self.tournaments = tournaments
        self.status = status
        self.privacy = privacy.value

    def to_json(self) -> JSONDict:
        return {
            "cover_image": self.cover_image,
            "bot_id": self.id,
            "name": self.name,
            "bio": self.bio,
            "link": self.link,
            "is_friend": self.is_friend,
            "user_name": self.username,
            "imageurl": self.image_url,
            "active": self.active,
            "deleted": self.deleted,
            "roleInfo": self.role_info,
            "admin": self.admin,
            "is_bot": self.is_bot,
            "is_game": self.is_game,
            "is_app": self.is_app,
            "app_callback": self.app_callback,
            "tournamentsParticipated": [x.to_json(x) for x in self.tournaments or []],
            "bot_privacy": self.privacy,
        }

    def from_json(self, data: Optional[JSONDict] = None) -> "User":

        if data is not None:
            self.id = int(data.get("id") or data.get("botId") or 0)
            self.name = data.get("name") or data.get("botName")
            self.bio = data.get("bio")
            self.is_friend = data.get("is_friend")
            self.is_game = data.get("is_game")
            self.link = data.get("link")
            self.username = data.get("user_name") or data.get("username")
            self.image_url = data.get("imageurl") or data.get("imageUrl")
            self.active = data.get("active")
            self.deleted = data.get("deleted")
            self.role_info = data.get("roleInfo")
            self.admin = data.get("admin")
            self.is_app = data.get("is_app")
            self.app_callback = data.get("app_callback")
            self.is_bot = data.get("is_bot") or data.get("bot")
            self.cover_image = data.get("cover_image")
            self.tournaments = UserTournament.build_from_json_list(
                data.get("tournamentsParticipated") or [], self.app
            )
            status = data.get("user_status", {}).get("status")
            if status:
                self.status = getattr(UserStatus, status, None)
            if privacy := data.get("bot_privacy"):
                self.privacy = getattr(BotPrivacy, privacy, None)
        return self



import swibots

from typing import List, Optional
from ..models import InstantMessaging
from swibots.api.community.models import CommunityMember, Community


class CommunityMethods:
    async def get_community_member(
        self: "swibots.ApiClient", community_id: str, user_id: str
    ) -> CommunityMember:
        """Get a community member.

        Args:
            community_id: The ID of the community.
            user_id: The ID of the user.

        Returns:
            The community member.
        """
        return await self.community_service.communities.get_community_member(
            community_id, user_id
        )

    async def get_community_members(
        self: "swibots.ApiClient", community_id: str
    ) -> List[CommunityMember]:
        """Get community members"""
        return await self.community_service.communities.get_community_members(
            community_id
        )

    async def create_community(self: "swibots.ApiClient", community: Community):
        """Create a community
        community: Community
        """
        return await self.community_service.communities.create_community(community)

    async def get_community(
        self: "swibots.ApiClient", id: str = "", username: str = ""
    ) -> Community:
        """Get a community by its ID or username

        Args:
            id (`str`): The ID of the community
            username (`str`): Username of the community

        Returns:
            :obj:`swibots.api.community.models.Community`: The community object.

        Raises:
            :obj:`switch.error.SwitchError`: If the community could not be retrieved
            :obj:`ValueError`: if the community id and username is provided together.

        This method does the same as :meth:`switch.api.community.controllers.CommunityController.get_community`.
        """
        return await self.community_service.communities.get_community(
            id, username=username
        )

    async def is_admin(
        self: "swibots.ApiClient", community_id: str, user_id: int
    ) -> bool:
        """Check if the user is admin of the community

        Args:
          community_id (str): Community ID
          user_id (int): User id
        """
        community_member = (
            await self.community_service.communities.get_community_member(
                community_id, user_id=user_id
            )
        )
        return community_member.admin

    async def get_active_commands(
        self: "swibots.ApiClient",
        community_id: str,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
    ):
        """Get Active commands in the community

        Args:
            community_id (str): Community ID
            channel_id (Optional[str], optional): Channel ID
            group_id (Optional[str], optional): Group ID. Defaults to None.

        Returns:
            _type_: _description_
        """
        return await self.community_service.communities.get_active_commands(
            community_id=community_id, group_id=group_id, channel_id=channel_id
        )

    async def approve_chat_join(
        self: "swibots.ApiClient",
        members: List[int],
        community_id: str,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
    ):
        """Approve private join request

        Args:
          members: List[user_id]
          community_id: Community ID
          group_id: the group id
          channel_id: the channel id
        """
        return await self.community_service.communities.approve_chat_join(
            members,
            community_id=community_id,
            group_id=group_id,
            channel_id=channel_id,
        )

    async def decline_chat_join(
        self: "swibots.ApiClient",
        members: List[int],
        community_id: str,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
    ):
        """Decline private join request

        Args:
          members: List[user_id]
          community_id: Community ID
          group_id: the group id
          channel_id: the channel id
        """
        return await self.community_service.communities.approve_chat_join(
            members,
            community_id=community_id,
            group_id=group_id,
            channel_id=channel_id,
            decline=True,
        )

    async def get_messaging_bots(
        self: "swibots.ApiClient",
        community_id: str,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
        bot_id: Optional[str] = None,
    ) -> List[InstantMessaging]:
        """Get bots having message handler permission in group or chanenl chat.

        Args:
            community_id (str): Community ID
            channel_id (Optional[str], optional): Channel ID.
            group_id (Optional[str], optional): Group ID.
            bot_id (Optional[str], optional): bot id.

        Returns:
            List[InstantMessaging]
        """
        return await self.community_service.communities.get_messaging_bots(
            community_id=community_id,
            group_id=group_id,
            channel_id=channel_id,
            bot_id=bot_id or self.user.id,
        )

    async def check_messaging_enabled(
        self,
        community_id: str,
        channel_id: Optional[str] = None,
        group_id: Optional[str] = None,
    ) -> bool:
        """Check whether instant messaging is enabled in current group or channel!

        Args:
            community_id (str): Community ID.
            channel_id (Optional[str], optional): Channel ID.
            group_id (Optional[str], optional): Group ID.

        Raises:
            ValueError: if none of channel_id or group_id is provided!

        Returns:
            bool: True if instant messaging is enabled!
        """
        if not (channel_id or group_id):
            raise ValueError(
                "'channel_id' or 'group_id' is required to check instant messaging!"
            )

        result = await self.get_messaging_bots(
            community_id=community_id, group_id=group_id, channel_id=channel_id
        )
        if not result:
            return
        return result[0].enabled

    async def update_community(
        self: "swibots.ApiClient", community: Community
    ) -> Community:
        """Update community

        Args:
            community (Community): New community info

        Returns:
            Community: new community response
        """
        return await self.community_service.communities.update_community(
            community=community
        )

    async def add_premium_member(
        self: "swibots.ApiClient",
        user_id: int,
        community_id: str,
        channel_id: str = None,
        group_id: str = None,
    ):
        """Add premium member to the community

        Args:
            user_id (int): User id
            community_id (str): Community id
            channel_id (str, optional): Channel id
            group_id (str, optional): Group id

        Returns:
            bool
        """
        return await self.community_service.communities.add_premium_member(
            user_id=user_id,
            community_id=community_id,
            channel_id=channel_id,
            group_id=group_id,
        )

    async def delete_premium_member(
        self: "swibots.ApiClient",
        user_id: int,
        community_id: str,
        channel_id: str = None,
        group_id: str = None,
    ):
        """Remove premium member to the community

        Args:
            user_id (int): User id
            community_id (str): Community id
            channel_id (str, optional): Channel id
            group_id (str, optional): Group id

        Returns:
            bool
        """
        return await self.community_service.communities.delete_premium_member(
            user_id=user_id,
            community_id=community_id,
            channel_id=channel_id,
            group_id=group_id,
        )

    async def get_premium_members(
        self: "swibots.ApiClient",
        user_id: int,
        community_id: str,
        channel_id: str = None,
        group_id: str = None,
    ):
        """Get all premium members in the community

        Args:
            user_id (int): User id
            community_id (str): Community id
            channel_id (str, optional): Channel id
            group_id (str, optional): Group id
        """
        return await self.community_service.communities.get_premium_members(
            user_id=user_id,
            community_id=community_id,
            channel_id=channel_id,
            group_id=group_id,
        )

    async def list_bots_in_community(self: "swibots.ApiClient", community_id: str):
        """List all bots in the community

        Args:
            community_id (str): Community ID
        """
        community_client = self.community_service.communities.client
        response = await community_client.get(
            f"/v1/community/bots?communityId={community_id}"
        )
        result = response.data["result"]
        members = result["communityMembers"]
        for member in members:

            def filterMemberId(x):
                return int(x["id"]) == int(member["userId"])

            userInfo = list(filter(filterMemberId, result["userInfoList"]))
            if userInfo:
                member["userInfo"] = userInfo[0]

        return community_client.build_list(CommunityMember, members)



import swibots
from typing import List

from swibots.api.community.models import Channel


class ChannelMethods:
    async def create_channel(self: "swibots.ApiClient", channel: Channel) -> str:
        """Create channel

        Args:
            :obj:`swibots.api.community.models.Channel`: The channel object.

        Returns:
            str:  channel id

        Raises:
            :obj:`switch.error.SwitchError`: If the channel could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.ChannelController.create_channel`.
        """
        return await self.community_service.channels.create_channel(channel)

    async def update_channel(self: "swibots.ApiClient", channel: Channel) -> str:
        """Update channel

        Args:
            :obj:`swibots.api.community.models.Channel`: The channel object.

        Returns:
            str:  channel id

        Raises:
            :obj:`switch.error.SwitchError`: If the channel could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.ChannelController.update_channel`.
        """
        return await self.community_service.channels.update_channel(channel)

    async def delete_channel(self: "swibots.ApiClient", id: str) -> bool:
        """Delete channel

        Args:
            :obj:`swibots.api.community.models.Channel`: The channel object.

        Returns:
            bool: Whether the channel was deleted!

        Raises:
            :obj:`switch.error.SwitchError`: If the channel could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.ChannelController.delete_channel`.
        """
        return await self.community_service.channels.delete_channel(id)

    async def get_all_channels(
        self: "swibots.ApiClient", community_id: str
    ) -> List[Channel]:
        """
        Get all channels from the community.
        """
        return await self.community_service.channels.get_all_channels(community_id)

    async def get_channel(self: "swibots.ApiClient", id: str) -> Channel:
        """Get a community by its ID

        Args:
            id (`str`): The ID of the channel

        Returns:
            :obj:`swibots.api.community.models.Channel`: The channel object.

        Raises:
            :obj:`switch.error.SwitchError`: If the channel could not be retrieved

        This method does the same as :meth:`switch.api.community.controllers.ChannelController.get_channel`.
        """
        return await self.community_service.channels.get_channel(id)



from swibots.utils.types import JSONDict
from .types import SwitchObject, Expansion, Component
from typing import List, Optional
from enum import Enum


class GridType(Enum):
    SMALL = "small"
    DEFAULT = "medium"
    LARGE = "large"


class GridItem(Component):
    type = "grid_tile"

    def __init__(
        self,
        title: str,
        media: str,
        dark_media: str = None,
        subtitle: str = "",
        callback_data: str = None,
        selective: bool = False,
    ):
        self.title = title
        self.subtitle = subtitle
        self.media = media
        self.dark_media = dark_media or media
        self.callback_data = callback_data
        self.selective = selective

    def to_json(self) -> JSONDict:
        return {
            "type": self.type,
            "selective": self.selective,
            "title": self.title,
            "subTitle": self.subtitle,
            "media": self.media,
            "callbackData": self.callback_data,
            "darkMedia": self.dark_media,
        }


class Grid(Component):
    type = "grid"

    def __init__(
        self,
        title: str = None,
        horizontal: bool = False,
        options: List[GridItem] = None,
        size: int = 3,
        expansion: Optional[Expansion] = Expansion.DEFAULT,
        right_image: str = None,
        image_callback: str = None,
        grid_type: GridType = GridType.DEFAULT,
    ):
        self.size = size
        self.options = options
        self.horizontal = horizontal
        self.title = title
        self.expansion = expansion
        self.right_image = right_image
        self.image_callback = image_callback
        self.grid_type = grid_type

    def to_json(self) -> JSONDict:
        data = {
            "type": self.type,
            "title": self.title,
            "children": [opt.to_json() for opt in self.options or []],
            "gridSize": self.size,
            "horizontalMode": self.horizontal,
            "expansion": self.expansion.value,
            "gridStyle": self.grid_type.value,
        }
        if self.right_image:
            data["subTitle"] = self.right_image
        if self.image_callback:
            data["subTitleAction"] = self.image_callback
        return data



import swibots
from swibots.utils.types import JSONDict
from swibots.base import SwitchObject

from .base_typed_inline_query_result import BaseTypedInlineQueryResult
from .input_message_content import InputMessageContent
from ..inline_markup import InlineMarkup
from .types import InlineQueryResultType


class InlineQueryResultPhoto(BaseTypedInlineQueryResult):
    def __init__(
        self,
        id: str,
        title: str,
        photo_url: str,
        mime_type: str = None,
        photo_width: int = None,
        photo_height: int = None,
        caption: str = None,
        input_message: "InputMessageContent" = None,
        description: str = None,
        thumb_url: str = None,
        thumb_width: int = None,
        thumb_height: int = None,
        reply_markup: "InlineMarkup" = None,
    ):
        super().__init__(
            id,
            InlineQueryResultType.PHOTO,
            title,
            description,
            thumb_url,
            thumb_width,
            thumb_height,
            input_message,
            reply_markup,
        )
        self.photo_url = photo_url
        self.photo_width = photo_width
        self.photo_height = photo_height
        self.caption = caption
        self.mime_type = mime_type

    def to_json(self) -> JSONDict:
        data = super().to_json()
        data.update(
            {
                "photoUrl": self.photo_url,
                "photoWidth": self.photo_width,
                "photoHeight": self.photo_height,
                "caption": self.caption,
                "mimeType": self.mime_type,
            }
        )
        return data
